from django.test import TestCase

from slumber import data_link
from slumber._caches import DJANGO_MODEL_TO_SLUMBER_MODEL

from slumber_examples.models import Pizza, Shop


class TestPizzaURIs(TestCase):
    def test_model_has_slumber_model(self):
        self.assertEqual(Pizza.slumber_model,
            DJANGO_MODEL_TO_SLUMBER_MODEL[Pizza])

    def test_instance_data_url(self):
        pizza = Pizza.objects.create(name="Test pizza")
        self.assertEqual(type(pizza).slumber_model.operations['data'](pizza.pk),
            '/slumber/slumber_examples/Pizza/data/%s/' % pizza.pk)

    def test_data_link(self):
        pizza = Pizza.objects.create(name="Test pizza")
        self.assertEqual(data_link(pizza),
            '/slumber/slumber_examples/Pizza/data/%s/' % pizza.pk)

    def test_empty_data_link(self):
        self.assertIsNone(data_link(None))

    def test_instance_data_url_when_passed_instance(self):
        pizza = Pizza.objects.create(name="Test pizza")
        self.assertEqual(type(pizza).slumber_model.operations['data'](pizza),
            '/slumber/slumber_examples/Pizza/data/%s/' % pizza.pk)


class TestShopURIs(TestCase):
    def test_model_has_slumber_model(self):
        self.assertEqual(Shop.slumber_model,
            DJANGO_MODEL_TO_SLUMBER_MODEL[Shop])

    def test_instance_data_uri(self):
        shop = Shop.objects.create(name="Test Cafe")
        self.assertEqual(type(shop).slumber_model.operations['data'](shop.pk),
            '/slumber/pizzas/shop/%s/' % shop.pk)

    def test_data_link(self):
        shop = Shop.objects.create(name="Test Cafe")
        self.assertEqual(data_link(shop), '/slumber/pizzas/shop/%s/' % shop.pk)

    def test_spaces_in_url_fragment(self):
        self.assertEqual(Shop.slumber_model.operations['shops1']('with space'),
            r'/slumber/shops/mount1/with%20space/')

    def test_query_string(self):
        self.assertEqual(Shop.slumber_model.operations['shops1'](key='v1'),
            r'/slumber/shops/mount1/?key=v1')

    def test_argument_which_re_roots(self):
        self.assertEqual(
            Shop.slumber_model.operations['shops1']('/shops/mount1'),
            r'/slumber/shops/mount1/')

    def test_argument_which_re_roots_all_the_way(self):
        self.assertEqual(
            Shop.slumber_model.operations['shops1']('/slumber/shops/mount1'),
            r'/slumber/shops/mount1/')



import copy
from datetime import date
from decimal import Decimal
import logging
import mock
import unittest2

from django.contrib.auth.models import User
from django.http import HttpResponse
import django.test

from slumber import client
from slumber.connector.ua import get, post
from slumber.test import mock_client, mock_ua

from slumber_examples.models import Order
from slumber_examples.tests.views import ServiceTestsWithDirectory


class TestSlumberMockClient(ServiceTestsWithDirectory, unittest2.TestCase):
    margarita = dict(pk=1, name='Margarita', for_sale=True)

    @mock_client(app__contrib__auth__Model=[], app__Pizza=[])
    def test_basic_app_data(self):
        """Ensure that the basic meta data part of the mock works as it should.
        """
        self.assertTrue(hasattr(client, 'app'))
        self.assertTrue(hasattr(client.app, 'contrib'))
        self.assertTrue(hasattr(client.app.contrib, 'auth'))
        self.assertTrue(hasattr(client.app.contrib.auth, 'Model'))
        self.assertTrue(hasattr(client.app, 'Pizza'))
        self.assertFalse(hasattr(client, 'slumber'))

    @mock_client(
        pizzas__slumber__Pizza=[
           margarita,
            dict(pk=2, name='Four seasons', prices=[
                dict(pk=2, amount=Decimal("13"))
            ]),
            dict(pk=3, name='Hawaiin', for_sale=False),
            dict(id=4, name='Diablo', for_sale=True),
        ],
        pizzas__slumber__PizzaPrice= [
            dict(pk=1,
                pizza=margarita,
                date='2010-01-01', amount=Decimal("14")),
        ])
    def test_get_pizza(self):
        """Ensure that we can manage instances and data arrays properly.
        """
        p1 = client.pizzas.slumber.Pizza.get(pk=1)
        self.assertEquals(p1.pk, 1)
        self.assertEquals(getattr(p1, 'name', None), 'Margarita', p1.__dict__)
        self.assertEquals(type(p1).__name__, "slumber://pizzas/slumber/Pizza/data/1/")

        p2 = client.pizzas.slumber.Pizza.get(pk=2)
        self.assertEquals(p2.pk, 2)
        self.assertEquals(len(p2.prices), 1)
        self.assertTrue(hasattr(p2.prices[0], 'pk'), type(p2.prices[0]))
        self.assertEquals(p2.prices[0].amount, Decimal("13"))

        pp1 = client.pizzas.slumber.PizzaPrice.get(pk=1)
        self.assertEquals(pp1.pk, 1)
        self.assertEquals(pp1.pizza.name, 'Margarita')

        p3 = client.pizzas.slumber.Pizza.get(id=4)
        self.assertEqual(type(p3).__name__, "slumber://pizzas/slumber/Pizza/data/4/")
        self.assertEqual(p3.name, "Diablo")
        with self.assertRaises(AttributeError):
            p3.prices

    @mock_client(pizzas__slumber__Pizza=[dict(pk = 3, name = 'Hawaiin', for_sale = False)])
    def test_update_pizza(self):
        """Ensure that we can update instances data properly."""

        Hawaiin_pizza =  client.pizzas.slumber.Pizza.get(pk = 3)
        self.assertEquals(Hawaiin_pizza.for_sale, False)

        client.pizzas.slumber.Pizza.update(Hawaiin_pizza, for_sale = True)

        Hawaiin_pizza =  client.pizzas.slumber.Pizza.get(pk = 3)
        self.assertEquals(Hawaiin_pizza.for_sale, True)

    @mock_client(pizzas__app__Model=[])
    def test_created_object_can_be_gotten(self):
        client.pizzas.app.Model.create(id=1, name='Test')
        item = client.pizzas.app.Model.get(id=1)
        self.assertEqual(item.name, 'Test')

    @mock_client(django__contrib__auth__User=[])
    def test_not_found_asserts(self):
        with self.assertRaises(AssertionError):
            client.django.contrib.auth.User.get(pk=1)

    @mock_client(pizzas__app__Model=[
        dict(pk=1)
    ])
    def test_alias_writes_are_visible(self):
        m1 = client.pizzas.app.Model.get(pk=1)
        m2 = client.pizzas.app.Model.get(pk=1)
        self.assertEqual(m1.pk, m2.pk)
        with self.assertRaises(AttributeError):
            m1.attr
        with self.assertRaises(AttributeError):
            m2.attr
        m1.attr = 'attribute data'
        self.assertEqual(m1.attr, 'attribute data')
        self.assertEqual(m1.attr, m2.attr)

    @mock_client(app__Model=[])
    def test_model_operation_url_is_correct(self):
        self.assertEqual(client.app.Model._operations['test-op'],
            'http://app/Model/test-op/')

    @mock_client(pizzas__app__Model=[
        dict(pk=1)
    ])
    def test_instance_operation_url_is_correct(self):
        model = client.pizzas.app.Model.get(pk=1)
        self.assertEqual(model._operations['test-op'],
            'http://pizzas/app/Model/test-op/1/')


class TestSlumberMockUA(ServiceTestsWithDirectory, unittest2.TestCase):
    @mock_ua
    def test_empty_mock_ua_with_no_activity(self, expect):
        pass

    @unittest2.expectedFailure
    @mock_ua
    def test_unmet_expectation(self, expect):
        expect.get('/', {})

    @unittest2.expectedFailure
    @mock_ua
    def test_no_get_expectation(self, expect):
        get('/')

    @unittest2.expectedFailure
    @mock_ua
    def test_get_with_wrong_query(self, expect):
        expect.get('/?query1', {})
        get('/?another')

    @unittest2.expectedFailure
    @mock_ua
    def test_no_post_expectation(self, expect):
        post('/', {})

    @unittest2.expectedFailure
    @mock_ua
    def test_no_expectated_get_got_post(self, expect):
        expect.get('/', {})
        post('/', {})

    @unittest2.expectedFailure
    @mock_ua
    def test_no_expectated_post_got_get(self, expect):
        expect.post('/', {}, {})
        get('/')

    @unittest2.expectedFailure
    @mock_ua
    def test_expect_wrong_post_data(self, expect):
        expect.post('/', {'data': True}, {})
        post('/', {'data': False})

    @mock_client(pizzas__app__Model=[])
    @mock_ua
    def test_model_operation_with_mock_ua(self, expect):
        expect.get('http://pizzas/app/Model/test-op/', {'test': 'item'})
        expect.post('http://pizzas/app/Model/test-op/', {'test': 'item'}, {'item': 'test'})
        self.assertEqual(len(expect.expectations), 2)
        response1, json1 = get(client.pizzas.app.Model._operations['test-op'])
        self.assertEqual(json1, dict(test='item'))
        response2, json2 = post(client.pizzas.app.Model._operations['test-op'], json1)
        self.assertEqual(json2, dict(item='test'))

    @mock_client(pizzas__app__Model=[])
    @mock_ua
    def test_model_operation_with_mock_ua_no_post_data_match(self, expect):
        expect.get('http://pizzas/app/Model/test-op/', {'test': 'item'})
        expect.post('http://pizzas/app/Model/test-op/', None, {'item': 'test'})
        self.assertEqual(len(expect.expectations), 2)
        response1, json1 = get(client.pizzas.app.Model._operations['test-op'])
        self.assertEqual(json1, dict(test='item'))
        response2, json2 = post(client.pizzas.app.Model._operations['test-op'], json1)
        self.assertEqual(json2, dict(item='test'))

    @mock_client(pizzas__django__contrib__auth__User=[
        dict(pk=1)])
    @mock_ua
    def test_instance_operation_with_mock_ua(self, expect):
        user = client.pizzas.django.contrib.auth.User.get(pk=1)
        expect.get(
            'http://pizzas/django/contrib/auth/User/has-permission/1/some.permission',
            {'is-allowed': True})
        has = user.has_perm('some.permission')
        self.assertTrue(has)


class TestMockWithDatabase(ServiceTestsWithDirectory, django.test.TestCase):
    def _setup(self):
        # Can't use setUp as that runs before the mock is applied
        self.shop = client.pizzas.slumber_examples.Shop.get(pk=1)
        self.order = Order(shop=self.shop)
        self.order.save()
        for order in Order.objects.all():
            logging.debug(order.shop)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_can_order_with_remote_shop_mock(self):
        self._setup()
        fetched = Order.objects.get(shop=self.shop)
        self.assertEqual(self.order, fetched)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_can_order_with_remote_shop_url(self):
        self._setup()
        fetched = Order.objects.get(
            shop='slumber://pizzas/slumber_examples/Shop/data/1/')
        self.assertEqual(self.order, fetched)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1, slug='test-shop-1')
    ])
    def test_order_shop_finds_mock(self):
        self._setup()
        self.assertEqual(self.order.shop.pk, 1)
        self.assertEqual(self.order.shop.slug, 'test-shop-1')

    @mock_client(pizzas__django__contrib__auth__User=[{
            'pk': 1, 'id': 1, 'username': 'test-user-1',
                'is_active': True, 'is_staff': False, 'is_superuser': False,
                'date_joined': date(2011, 05, 23),
                'first_name': 'test1', 'last_name': 'user', 'email': 'test-user-1@example.com'
        }, {
            'pk': 2, 'id': 2, 'username': 'test-user-2',
                'is_active': True, 'is_staff': False, 'is_superuser': False,
                'date_joined': date(2011, 05, 23),
                'first_name': 'test2', 'last_name': 'user', 'email': 'test-user-2@example.com'
        }])
    def test_model_proxy_is_applied(self):
        self.assertTrue(hasattr(client.pizzas.django.contrib.auth.User, 'authenticate'))
        def post(url, data):
            self.assertEqual(url,
                'http://pizzas/django/contrib/auth/User/authenticate/')
            return None, {'authenticated': True, 'user': {
                'url': 'slumber://pizzas/django/contrib/auth/User/data/2/',
                    'display_name': 'test2 user'}}
        with mock.patch('slumber.connector.proxies.post', post):
            user = client.pizzas.django.contrib.auth.User.authenticate()
            self.assertEqual(user, User.objects.get(username='test-user-2'))

    @mock_client(pizzas__django__contrib__auth__User=[{
            'pk': 1, 'id': 1, 'username': 'test-user-1',
                'is_active': True, 'is_staff': False, 'is_superuser': False,
                'date_joined': date(2011, 05, 23),
                'first_name': 'test1', 'last_name': 'user', 'email': 'test-user-1@example.com'
        }, {
            'pk': 2, 'id': 2, 'username': 'test-user-2',
                'is_active': True, 'is_staff': False, 'is_superuser': False,
                'date_joined': date(2011, 05, 23),
                'first_name': 'test2', 'last_name': 'user', 'email': 'test-user-2@example.com'
        }])
    def test_instance_proxy_is_applied_using_mocked_get(self):
        user = client.pizzas.django.contrib.auth.User.get(pk=2)
        self.assertTrue(hasattr(user, 'has_perm'), type(user).__mro__)


class TestViews(django.test.TestCase):
    @mock_client()
    def test_view(self):
        """Make sure that a simple view works with the mocked client.
        """
        self.client.get('/')

    @mock_client()
    @mock.patch('slumber_examples.views.ok_text')
    def test_mocked_then_patched(self, ok_text_patch):
        ok_text_patch.return_value = HttpResponse('patched')
        self.client.get('/')

    @mock.patch('slumber_examples.views.ok_text')
    @mock_client()
    def test_patched_then_mocked(self, ok_text_patch):
        ok_text_patch.return_value = HttpResponse('patched')
        self.client.get('/')


class TestCopy(ServiceTestsWithDirectory, django.test.TestCase):
    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop(self):
        shop = client.pizzas.slumber_examples.Shop.get(pk=1)
        deep = copy.deepcopy(shop)
        self.assertEqual(shop.pk, deep.pk)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop_in_order_query(self):
        query = Order.objects.filter(
            shop='slumber://pizzas/slumber_examples/Shop/data/1/')
        self.assertEqual(query.count(), 0)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop_in_order_query_double_filter_client_shop(self):
        shop = client.pizzas.slumber_examples.Shop.get(pk=1)
        query1 = Order.objects.filter(shop=shop)
        self.assertEqual(query1.count(), 0)
        query2 = query1.exclude(shop=shop)
        self.assertEqual(query2.count(), 0)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop_in_order_query_double_filter_shop_url(self):
        shop = 'slumber://pizzas/slumber_examples/Shop/data/1/'
        query1 = Order.objects.filter(shop=shop)
        self.assertEqual(query1.count(), 0)
        query2 = query1.exclude(shop=shop)
        self.assertEqual(query2.count(), 0)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_deepcopy_shop_in_order_query_double_filter_shop_from_order(self):
        order = Order.objects.create(
            shop = 'slumber://pizzas/slumber_examples/Shop/data/1/')
        query1 = Order.objects.filter(shop=order.shop)
        self.assertEqual(query1.count(), 1)
        query2 = query1.filter(shop=order.shop)
        self.assertEqual(query2.count(), 1)

    @mock_client(pizzas__slumber_examples__Shop = [
        dict(pk=1)
    ])
    def test_query_with_real_url(self):
        order = Order.objects.create(
            shop = 'slumber://pizzas/slumber_examples/Shop/data/1/')
        self.assertEqual(order.shop._url,
            'http://localhost:8000/slumber/pizzas/slumber_examples/Shop/data/1/')
        query = Order.objects.filter(shop=order.shop._url)
        self.assertEqual(query.count(), 1, [o.shop._url for o in list(Order.objects.all())])



"""Implements the conversion of the response data to valid xml data.
"""
from __future__ import absolute_import

import dicttoxml
from django.conf import settings
from django.http import HttpResponse
from xml.dom.minidom import parseString


def as_xml(_request, response, content_type):
    """Return http response object in XML format.
    """
    if hasattr(response, 'root'):
        xml = dicttoxml.dicttoxml(response[response.root], root=False)
    else:
        xml = dicttoxml.dicttoxml(response, root=False)
    xml = ('<?xml version="1.0" encoding="UTF-8" ?>\n'
                    '<{root}>{document}</{root}>'.format(
                root=getattr(response, 'root', 'root'),
                document=xml))
    if settings.DEBUG:
        xml = parseString(xml).toprettyxml()

    if content_type is not None and 'charset' not in content_type:
        content_type += '; charset=utf-8'

    return HttpResponse(xml, content_type,
        status=response['_meta']['status'])



"""
    Implements creation of an object.
"""
from slumber.operations import ModelOperation
from slumber.operations.instancedata import \
    instance_data, _reset_sequence_number
from slumber.server.http import require_permission
from slumber.server.json import to_json_data


class CreateInstance(ModelOperation):
    """Allows for the creation of new instances.
    """
    def post(self, request, response, appname, modelname):
        """Perform the object creation.
        """
        @require_permission('%s.add_%s' % (appname, modelname.lower()))
        def do_create(_cls, request):
            """Use an inner function so that we can generate a proper
            permission name at run time.
            """
            key_name = self.model.model._meta.pk.name
            if not request.POST.has_key(key_name):
                created = True
            else:
                filter_args = {key_name: request.POST[key_name]}
                objects = self.model.model.objects
                created = (objects.filter(**filter_args).count() == 0)

            instance = self.model.model(**dict([(k, v)
                for k, v in request.POST.items()]))
            instance.save()
            _reset_sequence_number(self.model.model)

            instance_data(response, self.model, instance)
            response['pk'] = to_json_data(self.model, instance, key_name,
                self.model.fields[key_name])
            response['created'] = created

        return do_create(self, request)



import logging
from mock import patch
from simplejson import dumps, loads

from django import get_version
from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.db import connection
from django.test import TestCase

from slumber import Client
from slumber.connector.ua import _calculate_signature, _fake_http_headers
from slumber.scheme import SlumberServiceURLError
from slumber_examples.models import Order, Pizza, PizzaPrice, Shop
from slumber_examples.tests.configurations import ConfigureUser


def _perform(client, method, url, data, content_type=None, username=None):
    def method_wrapper(*a, **kw):
        logging.info("Using wrapped GET handler")
        return client.get(*a, REQUEST_METHOD=method.upper(), **kw)
    logging.info("%s with data %s", method, data)
    headers = _calculate_signature('service',
        method.upper(), url, data, username)
    response = getattr(client, method, method_wrapper)(
        url, data, content_type=content_type, **_fake_http_headers(headers))
    if response.status_code == 200:
        return response, loads(response.content)
    else:
        return response, {}


class ViewTests(object):
    """Base class for view tests that give us some user agent functionality.
    """
    def do_get(self, url, query = {}, username=None):
        return _perform(self.client, 'get', self.url(url), query,
            username=username)

    def do_post(self, url, body):
        return _perform(self.client, 'post', self.url(url), dumps(body),
            'application/json')

    def do_delete(self, url):
        return _perform(self.client, 'delete', self.url(url), '')

    def do_options(self, url):
        return _perform(self.client, 'options', self.url(url), {})

    def url(self, path):
        if not path.startswith(self.PREFIX + '/'):
            return self.PREFIX + path
        else:
            return path


class PlainTests(object):
    """Used to get non-service based view tests.
    """
    PREFIX = '/slumber'


class ServiceTests(object):
    """Used to get service based view tests.
    """
    PREFIX  = '/slumber/pizzas'
    def patch(self, what, to):
        patcher = patch(what, to)
        self.__patchers.append(patcher)
        patcher.start()
    def setUp(self):
        self.__patchers = []
        self.patch('slumber.server._get_slumber_service', lambda: 'pizzas')
    def tearDown(self):
        [p.stop() for p in self.__patchers]


class ServiceTestsWithDirectory(ServiceTests):
    def setUp(self):
        super(ServiceTestsWithDirectory, self).setUp()
        directory = lambda: dict(pizzas='http://localhost:8000/slumber/pizzas/')
        self.patch('slumber.server._get_slumber_directory', directory)


class ViewErrors(ViewTests):

    def test_method_error(self):
        response, json = self.do_post('/slumber_examples/Pizza/instances/', {})
        self.assertEquals(response.status_code, 405)
        self.assertEquals(response['Allow'], 'GET, OPTIONS')

    def test_invalid_method(self):
        url = self.url('/slumber_examples/Pizza/instances/')
        response = self.client.get(url, REQUEST_METHOD='PURGE',
            HTTP_HOST='localhost', REMOTE_ADDR='127.0.0.1')
        self.assertEquals(response.status_code, 405, response.content)
        self.assertEquals(response['Allow'], 'GET, OPTIONS')

    def test_missing_slash(self):
        response, json = self.do_get('/slumber_examples')
        self.assertEquals(response.status_code, 301)
        self.assertTrue(response['location'].endswith('/slumber_examples/'),
            response['location'])

    def test_invalid_model(self):
        response, json = self.do_get('/slumber_examples/not-a-model/')
        self.assertEquals(response.status_code, 404)

    def test_invalid_model_operation(self):
        response, json = self.do_get('/slumber_examples/Pizza/not-an-operation/')
        self.assertEquals(response.status_code, 404)


class ViewErrorsPlain(ConfigureUser, ViewErrors, PlainTests, TestCase):
    pass
class ViewErrorsService(ConfigureUser, ViewErrors, ServiceTests, TestCase):
    def test_invalid_service(self):
        response = self.client.get('/slumber/not-a-service/')
        self.assertEquals(response.status_code, 404, response.content)


class BasicViews(ViewTests):
    def test_applications(self):
        self.assertTrue(bool(self.user))
        self.assertEqual(self.user.pk, 1)
        response, json = self.do_get('/')
        logging.debug(response)
        logging.debug(dumps(json, indent=4))
        apps = json['apps']
        self.assertEquals(apps['slumber_examples'], self.url('/slumber_examples/'))
        self.assertTrue(json.has_key('configuration'), json)
        self.assertTrue(json['configuration'].has_key('slumber_examples'), json)
        self.assertTrue(json['configuration']['slumber_examples']['test'], json)

    def test_model_search_success(self):
        response, json = self.do_get('/', {'model': 'slumber_examples.Pizza'})
        self.assertEquals(response.status_code, 302)
        self.assertTrue(response['location'].endswith(
            '/slumber_examples/Pizza/'), response['location'])

    def test_model_search_invalid(self):
        response, json = self.do_get('/', {'model': 'nota.model'})
        self.assertEquals(response.status_code, 404)

    def test_application_with_models(self):
        response, json = self.do_get('/slumber_examples/')
        self.assertEquals(response.status_code, 200)
        self.assertTrue(len(json['models']))
        self.assertEquals(json['models']['Pizza'],
            self.url('/slumber_examples/Pizza/'))

    def test_application_without_models(self):
        response, json = self.do_get('/slumber_examples/no_models/')
        self.assertEquals(response.status_code, 200)
        self.assertFalse(len(json['models']))

    def test_nested_application(self):
        response, json = self.do_get('/slumber_examples/nested1/')
        self.assertEquals(response.status_code, 200, 'slumber_examples.nested1')
    def test_doubly_nested_application(self):
        response, json = self.do_get('/slumber_examples/nested1/nested2/')
        self.assertEquals(response.status_code, 200, 'slumber_examples.nested1.nested2')
    def test_models_across_apps(self):
        response, json = self.do_get('/slumber_ex_shop/NestedModel/')
        self.assertEquals(response.status_code, 200, 'slumber_ex_shop.NestedModel')
        response, json = self.do_get('/slumber_ex_shop/Pizza/')
        self.assertEquals(response.status_code, 404, 'slumber_ex_shop.Pizza')

    def test_instance_metadata_pizza(self):
        response, json = self.do_get('/slumber_examples/Pizza/')
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['fields'].has_key('for_sale'))
        self.assertEquals(json['fields']['for_sale']['type'],
            'django.db.models.fields.BooleanField')
        self.assertEquals(json['operations']['instances'],
            self.url('/slumber_examples/Pizza/instances/'))
        self.assertFalse(json['operations'].has_key('data'), json['operations'])
        self.assertTrue(json['operations'].has_key('get'), json['operations'])

    def test_instance_metadata_pizzaprice(self):
        response, json = self.do_get('/slumber_examples/PizzaPrice/')
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['fields'].has_key('pizza'))
        self.assertEquals(json['fields']['pizza']['type'],
            self.url('/slumber_examples/Pizza/'))

    def test_model_metadata_user(self):
        response, json = self.do_get('/django/contrib/auth/User/')
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['operations'].has_key('authenticate'), json['operations'])
        self.assertEquals(json['operations']['authenticate'],
            self.url('/django/contrib/auth/User/authenticate/'))
        self.assertFalse(json['fields'].has_key('groups'), json['fields'].keys())
        self.assertIn('groups', json['data_arrays'])

    def test_instance_metadata_user(self):
        user = User(username='test-user')
        user.save()
        response, json = self.do_get('/django/contrib/auth/User/data/%s/' %
            user.pk)
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['operations'].has_key('has-permission'), json['operations'])

    def test_instance_puttable(self):
        response, json = self.do_get('/slumber_examples/Pizza/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(set([tuple(i) for i in json['puttable']]), set([('id',), ('name',)]))

    def test_model_operation_instances_no_instances(self):
        response, json = self.do_get('/slumber_examples/Pizza/instances/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 0)

    def test_model_operation_instances_one_instance(self):
        Pizza(name='S1', for_sale=True).save()
        response, json = self.do_get('/slumber_examples/Pizza/instances/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 1)

    def test_model_operation_instances_twelve_instances(self):
        for i in range(12):
            Pizza(name='S%s' % i, for_sale=True).save()
        response, json = self.do_get('/slumber_examples/Pizza/instances/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 10)
        self.assertEquals(json['next_page'],
            self.url('/slumber_examples/Pizza/instances/?start_after=3'))
        response, json = self.do_get('/slumber_examples/Pizza/instances/',
            {'start_after': '3'})
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 2)
        self.assertEquals(json['next_page'],
            self.url('/slumber_examples/Pizza/instances/?start_after=1'))
        response, json = self.do_get('/slumber_examples/Pizza/instances/',
            {'start_after': '1'})
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 0)
        self.assertFalse(json.has_key('next_page'), json)

    def test_instance_creation_get(self):
        response, json = self.do_get('/slumber_examples/Pizza/create/')
        self.assertEquals(response.status_code, 405, response.content)
        self.assertEquals(response['Allow'], 'OPTIONS, POST')

    def test_instance_creation_options(self):
        response, json = self.do_options('/slumber_examples/Pizza/create/')
        self.assertEquals(response.status_code, 200, response.content)
        self.assertEquals(response['Allow'], 'OPTIONS, POST')

    def test_instance_creation_post(self):
        self.user.is_superuser = True
        self.user.save()
        response, json = self.do_post('/slumber_examples/Pizza/create/',
            {'name': 'Test Pizza', 'for_sale': ''})
        self.assertTrue(json.get('identity', '').endswith(
            '/slumber_examples/Pizza/data/1/'), json)
        self.assertEquals(Pizza.objects.count(), 1)
        self.assertEquals(Pizza.objects.all()[0].name, 'Test Pizza')
        self.assertFalse(Pizza.objects.all()[0].for_sale)

    def test_update_instance(self):
        self.user.is_superuser = True
        self.user.save()
        s = Pizza(name='S1', for_sale=True)
        s.save()
        response, json = self.do_post('/slumber_examples/Pizza/update/1/', {
            'name': 'New pizza'})
        self.assertEquals(response.status_code, 200)
        n = Pizza.objects.get(pk=1)
        self.assertEquals(n.name, "New pizza")

    def test_get_instance(self):
        s = Pizza(name='S1', for_sale=True)
        s.save()
        response, json = self.do_get('/slumber_examples/Pizza/')
        get_url = json['operations']['get']
        self.assertEquals(get_url, self.url('/slumber_examples/Pizza/get/'))
        def check_query(query):
            response, json = self.do_get(get_url, query)
            self.assertEquals(response.status_code, 200, response)
            self.assertTrue(json['identity'].endswith(
                '/slumber_examples/Pizza/data/%s/' % s.pk), response)
        check_query({'pk': s.pk})
        check_query({'id': s.pk})
        check_query({'name': s.name})

    def test_instance_data_pizza(self):
        s = Pizza(name='S1', for_sale=True)
        s.save()
        response, json = self.do_get('/slumber_examples/Pizza/data/%s/' % s.pk)
        self.maxDiff = None
        expected = dict(
            _meta={'message': 'OK', 'status': 200, 'username': 'service'},
            type=self.url('/slumber_examples/Pizza/'),
            identity=self.url('/slumber_examples/Pizza/data/1/'),
            display='S1',
            operations=dict(
                data=self.url('/slumber_examples/Pizza/data/1/'),
                delete=self.url('/slumber_examples/Pizza/delete/1/'),
                order=self.url('/slumber_examples/Pizza/order/1/'),
                update=self.url('/slumber_examples/Pizza/update/1/')),
            fields=dict(
                id=dict(data=s.pk, kind='value', type='django.db.models.fields.AutoField'),
                for_sale=dict(data=s.for_sale, kind='value', type='django.db.models.fields.BooleanField'),
                max_extra_toppings=dict(data=s.max_extra_toppings, kind='value', type='django.db.models.fields.IntegerField'),
                name=dict(data=s.name, kind='value', type='django.db.models.fields.CharField'),
                exclusive_to={'data': None, 'kind': 'object', 'type': self.url('/slumber_examples/Shop/')}),
            data_arrays=dict(
                prices=self.url('/slumber_examples/Pizza/data/%s/prices/' % s.pk)))
        if get_version() >= "1.8":
            expected['fields']['exclusive_to_id'] = expected['fields']['exclusive_to']
        elif get_version() >= "1.7":
            expected['data_arrays']['exclusive_to_id'] = self.url(
                    '/slumber_examples/Pizza/data/1/exclusive_to_id/')
        self.assertEquals(json, expected)

    def test_instance_data_shop_with_null_active(self):
        s = Shop(name='Test shop', slug='test-shop')
        s.save()
        response, json = self.do_get('/slumber_examples/Shop/data/%s/' % s.pk)
        self.maxDiff = None
        expected = dict(
            _meta={'message': 'OK', 'status': 200, 'username': 'service'},
            type=self.url('/slumber_examples/Shop/'),
            identity=self.url('/slumber_examples/Shop/data/1/'),
            display='Test shop',
            operations=dict(
                data='/slumber/pizzas/shop/%s/' % s.pk,
                update=self.url('/slumber_examples/Shop/update/1/')),
            fields=dict(
                id=dict(data=s.pk, kind='value', type='django.db.models.fields.AutoField'),
                active=dict(data=None, kind='value', type='django.db.models.fields.NullBooleanField'),
                name={'data': 'Test shop', 'kind': 'value', 'type': 'django.db.models.fields.CharField'},
                slug={'data': 'test-shop', 'kind': 'value', 'type': 'django.db.models.fields.CharField'},
                web_address={'data': 'http://www.example.com/test-shop/', 'kind': 'property',
                    'type': 'slumber_examples.Shop.web_address'}),
            data_arrays={'pizza': self.url('/slumber_examples/Shop/data/1/pizza/')})
        self.assertEquals(json, expected)

    def test_instance_data_pizzaprice(self):
        s = Pizza(name='p1', for_sale=True)
        s.save()
        p = PizzaPrice(pizza=s, date='2010-01-01')
        p.save()
        response, json = self.do_get('/slumber_examples/PizzaPrice/data/%s/' % p.pk)
        self.assertEquals(response.status_code, 200)
        expected = dict(
            _meta={'message': 'OK', 'status': 200, 'username': 'service'},
            type=self.url('/slumber_examples/PizzaPrice/'),
            identity=self.url('/slumber_examples/PizzaPrice/data/1/'),
            display="p1 2010-01-01",
            operations=dict(
                data=self.url('/slumber_examples/PizzaPrice/data/1/'),
                delete=self.url('/slumber_examples/PizzaPrice/delete/1/'),
                update=self.url('/slumber_examples/PizzaPrice/update/1/')),
            fields=dict(
                id={'data': 1, 'kind': 'value', 'type': 'django.db.models.fields.AutoField'},
                pizza={'data': {
                        'type': self.url('/slumber_examples/Pizza/'), 'display':'p1',
                        'data': self.url('/slumber_examples/Pizza/data/1/')},
                    'kind': 'object', 'type': self.url('/slumber_examples/Pizza/')},
                date={'data': '2010-01-01', 'kind': 'value', 'type': 'django.db.models.fields.DateField'},
            ),
            data_arrays={'amounts': self.url('/slumber_examples/PizzaPrice/data/1/amounts/')})
        if get_version() >= "1.7":
            expected['data_arrays']['pizza_id'] = self.url(
                    '/slumber_examples/PizzaPrice/data/1/pizza_id/')
        self.assertEquals(json, expected)

    def test_instance_data_array(self):
        s = Pizza(name='P', for_sale=True)
        s.save()
        for p in range(15):
            PizzaPrice(pizza=s, date='2011-04-%s' % (p+1)).save()
        response, json = self.do_get('/slumber_examples/Pizza/data/%s/prices/' % s.pk)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 10, json)
        self.assertTrue(json.has_key('next_page'), json)
        self.assertEquals(json['next_page'],
            self.url('/slumber_examples/Pizza/data/1/prices/?start_after=6'),
            json['next_page'])
        response, json = self.do_get('/slumber_examples/Pizza/data/1/prices/',
            {'start_after': '6'})
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(json['page']), 5)
        self.assertEquals(json['page'][0], {
            'type': self.url('/slumber_examples/PizzaPrice/'),
            'pk': 5, 'data': self.url('/slumber_examples/PizzaPrice/data/5/'), 'display': 'P 2011-04-05'})
        self.assertFalse(json.has_key('next_page'), json.keys())

    def test_delete_instance_with_post(self):
        self.user.is_superuser = True
        self.user.save()
        s = Pizza(name='P')
        s.save()
        response, json = self.do_get('/slumber_examples/Pizza/data/%s/' % s.pk)
        self.assertEquals(response.status_code, 200)
        self.assertTrue(json['operations'].has_key('delete'), json['operations'])
        response, json = self.do_post(json['operations']['delete'], {})
        self.assertEquals(response.status_code, 200)
        with self.assertRaises(Pizza.DoesNotExist):
            Pizza.objects.get(pk=s.pk)

    def test_delete_instance_with_delete(self):
        self.user.is_superuser = True
        self.user.save()
        s = Pizza(name='P')
        s.save()
        response, json = self.do_get('/slumber_examples/Pizza/data/%s/' % s.pk)
        self.assertEquals(response.status_code, 200)
        response, _json = self.do_delete(json['operations']['data'])
        self.assertEquals(response.status_code, 200)
        with self.assertRaises(Pizza.DoesNotExist):
            Pizza.objects.get(pk=s.pk)
        response, _json = self.do_delete(json['operations']['data'])
        self.assertEquals(response.status_code, 404)



class BasicViewsPlain(ConfigureUser, BasicViews, PlainTests, TestCase):
    def test_service_configuration_missing_for_remoteforeignkey(self):
        self.user.is_superuser = True
        self.user.save()
        client = Client()
        shop = client.slumber_examples.Shop.create(name="Home", slug='home')
        order = Order(shop=shop)
        order.save()
        self.assertIsNotNone(order.shop)
        cursor = connection.cursor()
        cursor.execute(
            "SELECT shop FROM slumber_examples_order WHERE id=%s",
            [order.pk])
        row = cursor.fetchone()
        self.assertEquals(row[0], order.shop._url)
        with self.assertRaises(SlumberServiceURLError):
            order2 = Order.objects.get(pk=order.pk)


class BasicViewsService(ConfigureUser, BasicViews, ServiceTests, TestCase):
    def test_services_with_directory(self):
        with patch('slumber.server.get_slumber_directory', lambda: {
                'pizzas': 'http://localhost:8000:/slumber/pizzas/',
                'takeaway': 'http://localhost:8002:/slumber/'}):
            response = self.client.get('/slumber/',
                HTTP_HOST='localhost', REMOTE_ADDR='127.0.0.1')
        json = loads(response.content)
        self.assertEqual(response.status_code, 200, response.content)
        self.assertEqual(json['services'].get('pizzas', None),
            'http://localhost:8000:/slumber/pizzas/', json)
        self.assertEqual(json['services'].get('takeaway', None),
            'http://localhost:8002:/slumber/', json)

    def test_services_without_directory(self):
        response = self.client.get('/slumber/',
            HTTP_HOST='localhost', REMOTE_ADDR='127.0.0.1')
        json = loads(response.content)
        self.assertEqual(response.status_code, 200, response.content)
        self.assertTrue(json['services'].get('pizzas', '').endswith('/slumber/pizzas/'), json)

    def test_service_configuration_works_for_remoteforeignkey(self):
        self.user.is_superuser = True
        self.user.save()
        client = Client()
        shop = client.pizzas.slumber_examples.Shop.create(name="Home", slug='home')
        order = Order(shop=shop)
        order.save()
        self.assertIsNotNone(order.shop)
        cursor = connection.cursor()
        cursor.execute(
            "SELECT shop FROM slumber_examples_order WHERE id=%s",
            [order.pk])
        row = cursor.fetchone()
        self.assertEquals(row[0], 'slumber://pizzas/shop/1/')
        order2 = Order.objects.get(pk=order.pk)
        self.assertEquals(unicode(order2.shop), unicode(order.shop))
        self.assertEquals(order2.shop.id, order.shop.id)


class BasicViewsWithServiceDirectory(ConfigureUser, BasicViews,
        ServiceTestsWithDirectory, TestCase):
    def test_service_configuration_works_for_remoteforeignkey(self):
        self.user.is_superuser = True
        self.user.save()
        client = Client()
        shop = client.pizzas.slumber_examples.Shop.create(name="Home", slug='home')
        order = Order(shop=shop)
        order.save()
        cursor = connection.cursor()
        cursor.execute(
            "SELECT shop FROM slumber_examples_order WHERE id=%s",
            [order.pk])
        row = cursor.fetchone()
        self.assertEquals(row[0],
            'slumber://pizzas/shop/%s/' % shop.id)
        order2 = Order.objects.get(pk=order.pk)
        self.assertEquals(unicode(order2.shop), unicode(order.shop))
        self.assertEquals(order2.shop.id, order.shop.id)

    def test_instance_data_order(self):
        shop = Shop(name="Home", slug='home', active=True)
        shop.save()
        order = Order(shop=self.url("/slumber_examples/Shop/data/%s/" % shop.pk))
        order.save()
        response, json = self.do_get('/slumber_examples/Order/data/%s/' % order.pk)
        self.assertEquals(response.status_code, 200)
        expected = dict(
            _meta={'message': 'OK', 'status': 200, 'username': 'service'},
            type=self.url('/slumber_examples/Order/'),
            identity=self.url('/slumber_examples/Order/data/1/'),
            display="Order object",
            operations=dict(
                data=self.url('/slumber_examples/Order/data/1/'),
                delete=self.url('/slumber_examples/Order/delete/1/'),
                update=self.url('/slumber_examples/Order/update/1/')),
            fields=dict(
                id={'data': 1, 'kind': 'value', 'type': 'django.db.models.fields.AutoField'},
                nested={'data': None,
                        'kind': 'value',
                        'type': 'slumber.fields.RemoteForeignKey'},
                shop={'data': self.url('/slumber_examples/Shop/data/%s/' % shop.pk),
                      'kind': 'value',
                      'type': 'slumber.fields.RemoteForeignKey'}
            ),
            data_arrays={})
        self.assertEquals(json, expected)


class UserViews(ViewTests):
    authn = '/django/contrib/auth/User/authenticate/'
    data = '/django/contrib/auth/User/data/%s/'
    perm = '/django/contrib/auth/User/has-permission/%s/%s/'
    perms = '/django/contrib/auth/User/get-permissions/%s/'
    user_perm = '/django/contrib/auth/User/do-i-have-perm/%s/'
    user_perm_q = '/django/contrib/auth/User/do-i-have-perm/'

    def setUp(self):
        self.user = User(username='test-user')
        self.user.set_password('password')
        self.user.save()
        # The actual model doesn't matter so long as it is in auth
        self.content_type = ContentType.objects.get(model='user')
        super(UserViews, self).setUp()

    def test_user_data(self):
        response, json = self.do_get(self.data % self.user.pk)
        self.assertEqual(response.status_code, 200)
        self.assertIn('is_superuser', json['fields'].keys())
        self.assertIn('date_joined', json['fields'].keys())

    def test_user_not_found(self):
        response, json = self.do_post(self.authn, dict(username='not-a-user', password=''))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['authenticated'], False, json)
        self.assertIsNone(json['user'], json)

    def test_user_wrong_password(self):
        response, json = self.do_post(self.authn,
            dict(username=self.user.username, password='wrong'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['authenticated'], False, json)
        self.assertIsNone(json['user'], json)

    def test_user_authenticates(self):
        response, json = self.do_post(self.authn,
            dict(username=self.user.username, password='password'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['authenticated'], True, json)
        self.assertDictContainsSubset(
            {'pk': self.user.pk, 'display_name': 'test-user'},
            json['user'])
        self.assertTrue(
            json['user']['url'].endswith('/django/contrib/auth/User/data/3/'),
            json['user']['url'])

    def test_user_permission_no_permission(self):
        response, json = self.do_get(self.perm % (self.user.pk, 'foo.example'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['is-allowed'], False, json)

    def test_current_user_permission_no_permission(self):
        response, json = self.do_get(self.user_perm % 'foo.example',
            username=self.user.username)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['permissions']['foo.example'], False, json)

    def test_user_permission_is_allowed(self):
        permission = Permission(content_type=self.content_type,
            name='Can something', codename='can_something')
        permission.save()
        self.user.user_permissions.add(permission)
        response, json = self.do_get(self.perm % (self.user.pk, 'auth.can_something'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['is-allowed'], True, json)

    def test_current_user_permission_is_allowed(self):
        permission = Permission(content_type=self.content_type,
            name='Can something', codename='can_something')
        permission.save()
        self.user.user_permissions.add(permission)
        response, json = self.do_get(self.user_perm % 'auth.can_something',
            username=self.user.username)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['permissions']['auth.can_something'], True, json)

    def test_user_permission_not_allowed(self):
        permission = Permission(content_type=self.content_type,
            name='Can something', codename='can_something')
        permission.save()
        response, json = self.do_get(self.perm % (self.user.pk, 'auth.can_something'))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['is-allowed'], False, json)

    def test_current_user_permission_not_allowed(self):
        permission = Permission(content_type=self.content_type,
            name='Can something', codename='can_something')
        permission.save()
        response, json = self.do_get(self.user_perm % 'auth.can_something',
                username=self.user.username)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['permissions']['auth.can_something'], False, json)

    def test_current_user_multiple_permissions(self):
        can_perm = Permission(content_type=self.content_type,
            name='Can something', codename='can_something')
        can_perm.save()
        self.user.user_permissions.add(can_perm)
        cannot_perm = Permission(content_type=self.content_type,
            name='Cannot something', codename='cannot_something')
        cannot_perm.save()
        response, json = self.do_get(self.user_perm_q,
            dict(q=['foo.example', 'auth.can_something', 'auth.cannot_something']),
            username=self.user.username)
        self.assertEquals(response.status_code, 200)
        permissions = json['permissions']
        self.assertEquals(permissions['foo.example'], False, json)
        self.assertEquals(permissions['auth.can_something'], True, json)
        self.assertEquals(permissions['auth.cannot_something'], False, json)

    def test_current_user_no_permissions(self):
        response, json = self.do_get(self.user_perm_q,
            username=self.user.username)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(json['permissions'].keys(), [], json)

    def test_get_group_permissions(self):
        response, json = self.do_get(self.perms % self.user.pk)
        self.assertEquals(response.status_code, 200)
        self.assertItemsEqual(json['group_permissions'], [])


class UserViewsPlain(ConfigureUser, UserViews, PlainTests, TestCase):
    pass
class UserViewsService(ConfigureUser, UserViews, ServiceTests, TestCase):
    pass



"""
    Middleware to help manage the Slumber client.
"""
import logging

from slumber._caches import PER_THREAD


# Django defines the class members as methods
# pylint: disable=R0201


class Cache(object):
    """This middleware flushes the Slumber client cache at the start
    of each request.
    """

    def process_request(self, _request):
        """Turn the cache on.
        """
        logging.info("PER_THREAD instance cache created")
        PER_THREAD.cache = type('cache', (dict,), {})()

    def process_response(self, _request, response):
        """Turn the cache off again at the end of the request and flush it.
        """
        delattr(PER_THREAD, 'cache')
        logging.info("PER_THREAD instance cache removed")
        return response


class ForwardAuthentication(object):
    """Used to forward authentication of the currently logged in user to
    another backend.
    """

    def process_request(self, request):
        """Save the request in thread local storage so it can be retrieved
        by the user agent when it makes requests.
        """
        if request.user.is_authenticated():
            PER_THREAD.username = request.user.username

    def process_response(self, _request, response):
        """Forget the request, but do an assert to make sure nothing horrible
        has happened to it first.
        """
        PER_THREAD.username = None
        return response



from datetime import datetime
import logging
from mock import patch
from simplejson import loads

from django import get_version
from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponse
from django.test import TestCase

from fost_authn.authentication import FostBackend

from slumber import client
from slumber._caches import PER_THREAD
from slumber.connector.authentication import Backend, \
    ImproperlyConfigured
from slumber.connector.ua import  _sign_request, get
from slumber.test import mock_client
from slumber_examples.models import Pizza, Profile
from slumber_examples.tests.configurations import ConfigureUser, \
    ConfigureAuthnBackend, PatchForAuthnService


class TestAuthnRequired(ConfigureUser, TestCase):
    def setUp(self):
        self.pizza = Pizza(name='Before change')
        self.pizza.save()
        super(TestAuthnRequired, self).setUp()

    def test_directory_works_when_not_authenticated(self):
        response = self.client.get('/slumber/')
        self.assertEqual(response.status_code, 200)

    def test_model_requires_authentication(self):
        response = self.client.get('/slumber/slumber_examples/Pizza/data/%s/' % self.pizza.pk)
        self.assertEqual(response.status_code, 401)
        self.assertEqual(response.get('WWW-Authenticate', None),
            'FOST Realm="Slumber"')

    def test_authenticated(self):
        response = self.signed_get(None,
            '/slumber/slumber_examples/Pizza/data/234234/')
        self.assertEqual(response.status_code, 404)
        json = loads(response.content)
        self.assertTrue(
            json["error"].startswith("Pizza matching query does not exist."),
            json["error"])

    def test_model_create_requires_permission(self):
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/create/',
            {'name': 'new test pizza'})
        self.assertEqual(response.status_code, 403)

    def test_model_create_checks_correct_permission(self):
        permission = Permission.objects.get(codename="add_pizza")
        self.user.user_permissions.add(permission)
        response = self.signed_post(None,
            '/slumber/slumber_examples/Pizza/create/',
            {'name': 'new test pizza'})
        self.assertEqual(response.status_code, 200, response.content)

    def test_model_update_requires_permission(self):
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/update/%s/' % self.pizza.pk,
            {'name': 'new test pizza'})
        self.assertEqual(response.status_code, 403)

    def test_model_update_checks_correct_permission(self):
        permission = Permission.objects.get(codename="change_pizza")
        self.user.user_permissions.add(permission)
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/update/%s/' % self.pizza.pk,
            {'name': 'new test pizza'})
        self.assertEqual(response.status_code, 200, response.content)

    def test_model_delete_requires_permission(self):
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/delete/%s/' % self.pizza.pk, {})
        self.assertEqual(response.status_code, 403)

    def test_model_delete_checks_correct_permission(self):
        permission = Permission.objects.get(codename="delete_pizza")
        self.user.user_permissions.add(permission)
        response = self.signed_post(self.user.username,
            '/slumber/slumber_examples/Pizza/delete/%s/' % self.pizza.pk, {})
        self.assertEqual(response.status_code, 200, response.content)


class TestAuthnForwarding(ConfigureUser, TestCase):
    def setUp(self):
        settings.MIDDLEWARE_CLASSES.append(
            'slumber.connector.middleware.ForwardAuthentication')
        super(TestAuthnForwarding, self).setUp()
    def tearDown(self):
        super(TestAuthnForwarding, self).tearDown()
        settings.MIDDLEWARE_CLASSES.remove(
            'slumber.connector.middleware.ForwardAuthentication')

    def test_signing_function_signs(self):
        headers = {}
        def check_request(request):
            for k, v in _sign_request('GET', '/', '').items():
                headers[k] = v
            return HttpResponse('ok', 'text/plain')
        with patch('slumber_examples.views._ok_text', check_request):
            self.client.get('/', REMOTE_ADDR='127.0.0.1')
        self.assertTrue(headers.has_key('Authorization'), headers)

    def test_username_with_colon(self):
        self.user.username = "my:name"
        self.user.save()
        headers = {}
        def check_request(request):
            for k, v in _sign_request('GET', '/', '').items():
                headers[k] = v
            return HttpResponse('ok', 'text/plain')
        with patch('slumber_examples.views._ok_text', check_request):
            self.signed_get(self.user.username)
        self.assertTrue(headers.has_key('Authorization'), headers)
        self.assertTrue(headers.has_key('X-FOST-User'), headers)
        self.assertEqual(headers['X-FOST-User'], self.user.username)

    def test_username_with_unicode(self):
        self.user.username = u"my\u2014name" # 0x2014 is mdash
        self.user.save()
        headers = {}
        def check_request(request):
            for k, v in _sign_request('GET', '/', '').items():
                headers[k] = v
            return HttpResponse('ok', 'text/plain')
        with patch('slumber_examples.views._ok_text', check_request):
            self.signed_get(self.user.username)
        self.assertTrue(headers.has_key('Authorization'), headers)
        self.assertTrue(headers.has_key('X-FOST-User'), headers)
        self.assertEqual(
            headers['X-FOST-User'].decode('utf-7'), self.user.username)

    def test_authentication_backend_accepts_signature(self):
        def check_request(request):
            class response:
                status = 200
                content = '''null'''
            def _request(_self, url, headers={}):
                backend = FostBackend()
                authz = headers['Authorization']
                key = authz[5:5+len(self.service.username)]
                signature = authz[6+len(self.service.username):]
                logging.info('Authorization %s %s', key, signature)
                request.META['HTTP_X_FOST_TIMESTAMP'] = headers[
                    'X-FOST-Timestamp']
                request.META['HTTP_X_FOST_HEADERS'] = headers[
                    'X-FOST-Headers']
                user = backend.authenticate(request=request,
                    key=key, hmac=signature)
                self.assertTrue(user)
                r = response()
                return r, r.content
            with patch('slumber.connector.ua.Http.request', _request):
                response, json = get('http://example.com/')
            return HttpResponse(response.content, 'text/plain')
        with patch('slumber_examples.views._ok_text', check_request):
            self.signed_get(self.user.username)


class TestBackend(PatchForAuthnService, TestCase):
    def setUp(self):
        super(TestBackend, self).setUp()
        self.backend = Backend()

    def test_remote_user(self):
        user = client.auth.django.contrib.auth.User.get(username='user')
        for attr in ['is_active', 'is_staff', 'date_joined', 'is_superuser',
                'first_name', 'last_name', 'email', 'username']:
            self.assertTrue(hasattr(user, attr), user.__dict__.keys())

    def test_remote_login(self):
        user = self.backend.authenticate(
            username=self.user.username, password='pass')
        self.assertTrue(user)
        self.assertEqual(user.username, self.user.username)

    def test_remote_login_with_unicode_username(self):
        # 2014 - mdash, 203d - interrobang
        self.user.username = u'interesting\u2014user\u203d'
        self.user.save()
        user = self.backend.authenticate(
            username=self.user.username, password='pass')
        self.assertTrue(user)
        self.assertEqual(user.username, self.user.username)

    def test_remote_login_with_unicode_password(self):
        # 203d - interrobang
        self.user.set_password(u'newpass\u203d')
        self.user.save()
        user = self.backend.authenticate(
            username=self.user.username, password=u'newpass\u203d')
        self.assertTrue(user)
        self.assertEqual(user.username, self.user.username)

    def test_remote_login_with_wrong_password(self):
        user = self.backend.authenticate(username=self.user.username, password='xxxx')
        self.assertIsNone(user)

    def test_login_type_not_recognised(self):
        user = self.backend.authenticate(made_up=True)
        self.assertIsNone(user)

    def test_get_user(self):
        user = self.backend.get_user(self.user.username)
        self.assertEqual(user.username, self.user.username)
        self.assertTrue(user.is_active)
        self.assertTrue(user.is_staff)
        self.assertFalse(user.is_superuser)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertEqual(user.username, user.remote_user.username)
        self.assertEqual(user.is_active, user.remote_user.is_active)
        self.assertEqual(user.is_staff, user.remote_user.is_staff)
        self.assertEqual(user.is_superuser, user.remote_user.is_superuser)

    def test_get_user_with_id(self):
        user = self.backend.get_user(self.user.pk)
        self.assertTrue(user)

    def test_cache_ttl(self):
        user = self.backend.get_user(self.user.username)
        self.assertEqual(user.remote_user._CACHE_TTL, 120)

    def test_group_permissions(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        perms = self.backend.get_group_permissions(user)
        self.assertEqual(perms, self.user.get_group_permissions())

    def test_all_permissions(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        perms = self.backend.get_all_permissions(user)
        self.assertEqual(perms, self.user.get_all_permissions())

    def test_module_perms(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_module_perms(user, 'slumber_examples'))

    def test_existing_permission(self):
        self.assertTrue(bool(ContentType.objects.all().count()))
        content_type = ContentType.objects.get(
            app_label='slumber_examples', model='pizza')
        permission = Permission.objects.get(
            codename='add_pizza', content_type=content_type)
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_perm(user, 'slumber_examples.add_pizza'))

    def test_existing_permission(self):
        self.assertTrue(bool(ContentType.objects.all().count()))
        content_type = ContentType.objects.get(
            app_label='slumber_examples', model='pizza')
        permission = Permission.objects.get(
            codename='add_pizza', content_type=content_type)
        self.user.user_permissions.add(permission)
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertTrue(self.backend.has_perm(user, 'slumber_examples.add_pizza'))

    def test_missing_permission(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_perm(user, 'slumber_examples.not-a-perm'))
        perm = Permission.objects.get(codename='not-a-perm',
            content_type__app_label='slumber_examples')
        self.assertEqual(perm.codename, 'not-a-perm')
        self.assertEqual(perm.name, perm.codename)
        self.assertEqual(perm.content_type.app_label, 'slumber_examples')
        self.assertEqual(perm.content_type.model, 'unknown')

    def test_permission_with_new_app(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_perm(user, 'not-an-app.not-a-perm'))
        perm = Permission.objects.get(codename='not-a-perm',
            content_type__app_label='not-an-app')
        self.assertEqual(perm.codename, 'not-a-perm')
        self.assertEqual(perm.name, perm.codename)
        self.assertEqual(perm.content_type.app_label, 'not-an-app')
        self.assertEqual(perm.content_type.model, 'unknown')

    def test_permission_with_invalid_name(self):
        user = self.backend.get_user(self.user.username)
        self.assertTrue(hasattr(user, 'remote_user'))
        self.assertFalse(self.backend.has_perm(user, 'not-a-perm'))
        self.assertFalse(self.backend.has_perm(user, 'not-an-app..not-a-perm'))

    def test_user_profile_when_no_profile(self):
        if get_version() < "1.7":
            user = self.backend.get_user(self.user.username)
            with self.assertRaises(AssertionError):
                profile = user.get_profile()

    def test_user_profile_when_there_is_a_profile(self):
        if get_version() < "1.7":
            profile = Profile(user=self.user)
            profile.save()
            user = self.backend.get_user(self.user.username)
            remote_profile = user.get_profile()
            self.assertEqual(remote_profile.id, profile.id)
            self.assertEqual(remote_profile.user.id, self.user.id)


class AuthenticationTests(ConfigureAuthnBackend, TestCase):
    def save_user(self, request):
        self.user = request.user
        return HttpResponse('ok')

    @mock_client(
        django__contrib__auth__User = []
    )
    def test_improperly_configured(self):
        with self.assertRaises(ImproperlyConfigured):
            self.signed_get('testuser')

    @mock_client(
        auth__django__contrib__auth__User = [
            dict(username='testuser', is_active=True, is_staff=True,
                date_joined=datetime.now(), is_superuser=False,
                    first_name='Test', last_name='User',
                    email='test@example.com')],
    )
    def test_is_authenticated(self):
        with patch('slumber_examples.views._ok_text', self.save_user):
            self.signed_get('testuser')
        self.assertTrue(self.user.is_authenticated())
        self.assertEqual(self.user.username, 'testuser')

    @mock_client(
        auth__django__contrib__auth__User = [
            dict(username='testuser', is_active=True, is_staff=False,
                date_joined=datetime.now(), is_superuser=False,
                    first_name='Test', last_name='User',
                    email='test@example.com')],
    )
    def test_created_user_sees_changes(self):
        self.signed_get('testuser')
        remote_user = client.auth.django.contrib.auth.User.get(
            username='testuser')
        remote_user.is_staff = True
        with patch('slumber_examples.views._ok_text', self.save_user):
            self.signed_get('testuser')
        self.assertTrue(self.user.is_staff)

    @mock_client(
        auth__django__contrib__auth__User = [
            dict(username='admin', is_active=True, is_staff=True,
                date_joined=datetime.now(), is_superuser=False,
                    first_name='Test', last_name='User',
                    email='test@example.com')],
    )
    def test_admin_is_authenticated(self):
        admin = User(username='admin')
        admin.save()
        with patch('slumber_examples.views._ok_text', self.save_user):
            self.signed_get(admin.username)
        self.assertTrue(self.user.is_authenticated())
        self.assertEqual(admin, self.user)

    @mock_client(
        auth__django__contrib__auth__User = []
    )
    def test_remote_user_not_found(self):
        with patch('slumber_examples.views._ok_text', self.save_user):
            self.client.get('/', HTTP_X_FOST_USER='testuser')
        self.assertFalse(self.user.is_authenticated())



from __future__ import absolute_import

import dicttoxml
from xml.dom.minidom import parseString

from django.conf import settings
from django.http import HttpResponse
from django.test import TestCase

from slumber.server.xml import as_xml
from slumber_examples.tests import ConfigureUser


class TestXML(ConfigureUser, TestCase):
    def setUp(self):
        super(TestXML, self).setUp()
        self.old_value = settings.DEBUG
        settings.DEBUG = True

    def tearDown(self):
        settings.DEBUG = self.old_value
        super(TestXML, self).tearDown()

    def test_as_xml(self):
        # Arrange
        request = {}
        response = {'_meta': dict(status=200, message='OK'),
                    'fake_content': 'sputnik'}
        content_type = 'text/xml'

        xml_snippet = dicttoxml.dicttoxml(response, root=True)
        dom = parseString(xml_snippet).toprettyxml()

        expected_content_type = "%s; charset=utf-8" % content_type
        expected_response = HttpResponse(dom, expected_content_type, 200)

        # Act
        http_response = as_xml(request, response, content_type)

        # Assert
        self.assertEqual(str(http_response), str(expected_response))


    def test_as_xml_should_not_append_charset_if_its_provided(self):
        # Arrange
        request = {}
        response = {'_meta': dict(status=200, message='OK'),
                    'fake_content': 'sputnik'}
        content_type = 'text/xml; charset=utf-16'

        # Act
        http_response = as_xml(request, response, content_type)

        # Assert
        self.assertEqual(http_response['Content-Type'], 'text/xml; charset=utf-16')



from slumber.operations import InstanceOperation, ModelOperation


class OrderPizza(InstanceOperation):
    def get(self, request, response, app, model, pk):
        response['form'] = dict(quantity='integer')

    def post(self, _request, _response, _app, _model, _pk):
        raise NotImplementedError("OrderPizza.post")


class ShopList(ModelOperation):
    def get(self, _request, response, _app, _model):
        response['shops'] = [{
                'name': 'Hard Coded Pizza Parlour'
            }]




"""
    Implements the server side operations on models and instances.
"""
from urllib import quote, urlencode

from django.core.urlresolvers import reverse
from django.db.models import Model

from slumber.server import get_slumber_root


def _forbidden(_request, response, *_):
    """Return an error to say that the method type is not allowed.
    """
    response['_meta']['status'] = 405
    response['_meta']['message'] = "Method Not Allowed"


class Operation(object):
    """Base class for operations. The ModelOperation and InstanceOperation
    classes are for backward compatibility, but their use is deprecated.
    """
    METHODS = ['GET', 'OPTIONS', 'POST', 'PUT', 'DELETE']

    def __init__(self, model, name, uri=None):
        self.model = model
        self.name = name
        self.uri = uri
        self.regex = ''
        self.path = model.path + name + '/'

    def __call__(self, *args, **qs):
        root = get_slumber_root()
        uri = self.uri or (root + self.path)
        for part in args:
            if issubclass(type(part), Model):
                part = part.pk
            part = str(part)
            if part.startswith('/'):
                if part.startswith(root):
                    uri = part
                else:
                    uri = root + part[1:]
            else:
                uri += quote(part)
            if not uri.endswith('/'):
                uri += '/'
        if qs:
            uri += '?' + urlencode(qs)
        return uri

    def headers(self, retvalue, request, response):
        """Calculate and place extra headers needed for certain types of
        response.
        """
        if response['_meta']['status'] == 405 or request.method == 'OPTIONS':
            response['_meta'].setdefault('headers', {})
            response['_meta']['headers']['Allow'] = \
                ', '.join([method
                    for method in self.METHODS
                        if hasattr(self, method.lower())])
        return retvalue

    def operation(self, request, response, _appname, _model, *args):
        """Perform the requested operation in the server.
        """
        if request.method in self.METHODS:
            retvalue = getattr(self, request.method.lower(), _forbidden)(
                request, response, *args)
            return self.headers(retvalue, request, response)
        else:
            _forbidden(request, response)
        return self.headers(None, request, response)

    def options(self, request, response, *_):
        """A standard options response that will fill in the Allow header.
        """
        return self.headers(None, request, response)


class ModelOperation(Operation):
    """Base class for model operations.
    """
    model_operation = True

    def operation(self, request, response, *args):
        """Perform the requested operation in the server.
        """
        if request.method in self.METHODS:
            retvalue = getattr(self, request.method.lower(), _forbidden)(
                request, response, *args)
            return self.headers(retvalue, request, response)
        else:
            _forbidden(request, response)
        return self.headers(None, request, response)


class InstanceOperation(ModelOperation):
    """Base class for operations on instances.
    """
    model_operation = False
    def __init__(self, model, name, uri=None):
        super(InstanceOperation, self).__init__(model, name, uri)
        self.regex = '([^/]+)/'




"""
    Implements a listing of all instances for a given model.
"""
from dougrain import Builder

from slumber.operations import ModelOperation
from slumber.server import get_slumber_root
from slumber.server.http import require_user


class InstanceList(ModelOperation):
    """Allows access to the instances.
    """
    @require_user
    def get(self, request, response, _appname, _modelname):
        """Return a paged set of instances for this model.
        """
        root = get_slumber_root()
        response['model'] = root + self.model.path

        query = self.model.model.objects.order_by('-pk')
        if request.GET.has_key('start_after'):
            query = query.filter(pk__lt=request.GET['start_after'])

        response['page'] = [
                dict(pk=o.pk, display=unicode(o),
                    data=self.model.operations['data'](o))
            for o in query[:10]]
        if len(response['page']) > 0:
            response['next_page'] = self(
                start_after=response['page'][-1]['pk'])


def hal_instance_list(operation, control, builder, query_set, page_size=10):
    """Return a page of JSON-HAL based results across the query set.
    """
    from slumber import data_link
    starter = control.get('lpk', None)
    if starter:
        query_set = query_set.filter(pk__lt=starter)
    lpk = None
    for instance in query_set.order_by('-pk').iterator():
        if page_size == 0:
            builder.add_link('next', operation(lpk=lpk))
            return
        else:
            page_size -= 1
            lpk = instance.pk
            item = Builder(data_link(instance))
            item.set_property('display', unicode(instance))
            builder.embed('page', item)


class InstanceListHal(ModelOperation):
    """Allow us to get an instance list in HAL format.
    """
    @require_user
    def get(self, request, response, _appname, _modelname):
        """Return a HAL formatted version of the instance list.
        """
        root = get_slumber_root()

        hal = Builder(self())
        hal.add_link('model', root + self.model.path)

        query = self.model.model.objects
        hal_instance_list(self, request.GET, hal, query)

        response["instances"] = hal.as_object()
        response.root = 'instances'



"""
    The Django URLs for the server.
"""
try:
    # This API has changed.
    # pylint: disable=no-name-in-module
    from django.conf.urls import url
except ImportError:
    # pylint: disable=no-name-in-module, import-error
    from django.conf.urls.defaults import url


# The name urlpatterns is defined by Django and we can't change it
# pylint: disable=C0103
urlpatterns = [
    url('.*', 'slumber.server.views.service_root')
]



from django.contrib import admin
from slumber_examples.models import (Pizza, PizzaCrust, PizzaPrice,
    PizzaSizePrice, Shop)


admin.site.register(Pizza)
admin.site.register(PizzaCrust)
admin.site.register(PizzaPrice)
admin.site.register(PizzaSizePrice)
admin.site.register(Shop)



"""
    Some caches used in the implementation of the Slumber client or server.
"""
import threading


# Stores the applications via their application names
APP_FROM_APPNAME = {}

# Stores the server model for a given Django model in the server
DJANGO_MODEL_TO_SLUMBER_MODEL = {}
# Stores the slumber models for given model URLs
MODEL_URL_TO_SLUMBER_MODEL = {}
# Stores the slumber model for a given database table name
DATABASE_TABLE_TO_SLUMBER_MODEL = {}

# Store the URIs for special operations
OPERATION_URIS = {}


# Add a location where we can save per thread data
PER_THREAD = threading.local()



from mock import patch, Mock
from simplejson import loads, dumps
from django.http import HttpResponse
from django.test import TestCase
from slumber.server import accept_handler
from slumber.server.http import view_handler
from slumber.server import xml
from slumber.server import html
from django.conf import settings


class TestAcceptHandler(TestCase):
    def setUp(self):
        class Request(object):
            META = {'HTTP_ACCEPT': 'application/xml;', 'CONTENT_TYPE': None}
        self.request = Request()
        self.response = {'_meta': {'status': 200}}

    def test_accept_handler(self):
        content_type, handler = accept_handler.accept(self.request.META['HTTP_ACCEPT'])
        expect_http_response = handler(self.request, self.response, self.request.META.get('CONTENT_TYPE', None))
        self.assertTrue(isinstance(expect_http_response, HttpResponse))

    def test_accept_handler_with_content(self):
        test_str = {'test_value': 'hello'}
        accept_handlers_list = [
            ('application/json', lambda req, res, ct: HttpResponse(dumps(test_str), 'text/plain', status=200))
        ]
        fake_accept_str = 'text/*; application/json;'
        content_type, handler = accept_handler.accept(fake_accept_str, accept_handlers_list)
        expect_http_response = handler(self.request, self.response, self.request.META.get('CONTENT_TYPE', None))
        self.assertTrue(isinstance(expect_http_response, HttpResponse))
        self.assertEqual(loads(expect_http_response.content), test_str)

    def test_accept_handler_with_mix_value(self):
        accept_handlers_list = [
            ('application/json', lambda req, res, ct: None),
            ('application/xhtml', lambda req, res, ct: HttpResponse(dumps({'fake_fn2': True}), 'text/plain', status=200))
        ]
        fake_accept_str = 'application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5'
        content_type, handler = accept_handler.accept(fake_accept_str, accept_handlers_list)
        expect_http_response = handler(self.request, self.response, self.request.META.get('CONTENT_TYPE', None))
        self.assertTrue(isinstance(expect_http_response, HttpResponse))
        self.assertEqual(loads(expect_http_response.content), {'fake_fn2': True})


class TestUsingAcceptHandler(TestCase):
    def setUp(self):
        class Request(object):
            META = {'HTTP_ACCEPT': 'application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;'}
            class user(object):
                @classmethod
                def is_authenticated(cls):
                    return True
                username = 'testuser'
        self.test_request = Request

    @patch('slumber.server.accept_handler.get_handlers_list')
    def test_with_accept_handler_with_default(self, mock_handler_list):
        settings.DEBUG = False
        test_str = 'just a fake content'
        mock_handler_list.return_value = []
        test_result_str = '{"_meta": {"status": 200, "username": "testuser", "message": "OK"}, "fake_content": "just a fake content"}'


        @view_handler
        def view(request, response):
            response['fake_content'] = test_str

        http_response = view( self.test_request() )

        self.assertEqual(test_result_str, str(http_response.content))

    @patch('slumber.server.accept_handler.get_handlers_list')
    def test_with_accept_header_value(self, mock_handler_list):
        test_str = 'just a fake content'

        mock_handler_list.return_value = [
            ('application/xml', lambda req, res, ct: HttpResponse(dumps(res), 'text/plain')),
        ]

        @view_handler
        def view(request, response):
            response['fake_content'] = test_str

        http_response = view( self.test_request() )
        content = loads(http_response.content)
        self.assertEquals(content, dict(
            fake_content=str(test_str),
            _meta=dict(status=200, message="OK", username="testuser")))

    @patch('slumber.server.accept_handler.get_handlers_list')
    def test_with_accept_header_as_xml(self, mock_handler_list):
        xml.as_xml = Mock()
        test_str = 'just a fake content'

        mock_handler_list.return_value = [
            ('application/json', lambda req, res, ct: HttpResponse(dumps(res), 'text/plain')),
            ('application/xml', xml.as_xml),
        ]

        @view_handler
        def view(request, response):
            response['fake_content'] = test_str

        view(self.test_request())
        self.assertTrue(xml.as_xml.called)

    @patch('slumber.server.accept_handler.get_handlers_list')
    def test_with_accept_handler_as_html(self, mock_handler_list):
        html.build_html = Mock()
        test_str = 'just a fake content'

        mock_handler_list.return_value = [
            ('application/json', lambda req, res, ct: HttpResponse(dumps(res), 'text/plain')),
            ('text/html', html.build_html),
        ]

        @view_handler
        def view(request, response):
            response['fake_content'] = test_str

        view( self.test_request() )
        self.assertTrue(html.build_html.called)



"""
    Implements the JSON formatting for the server.
"""
from simplejson import dumps, JSONEncoder

from django.http import HttpResponse
from django.conf import settings

from slumber._caches import DJANGO_MODEL_TO_SLUMBER_MODEL
from slumber.server import get_slumber_root


DATA_MAPPING = {
        'django.db.models.fields.AutoField': lambda m, i, fm, v: v,
        'django.db.models.fields.BooleanField': lambda m, i, fm, v: v,
        'django.db.models.fields.NullBooleanField': lambda m, i, fm, v: v,
        'slumber.fields.RemoteForeignKey':
            lambda m, i, fm, v: v._url if v else None,
    }


def to_json_data(model, instance, fieldname, fieldmeta):
    """Convert a model field to JSON on the server.
    """
    value = getattr(instance, fieldname)
    if fieldmeta['kind'] == 'object':
        if value is None:
            return None
        else:
            rel_to = DJANGO_MODEL_TO_SLUMBER_MODEL[type(value)]
            root = get_slumber_root()
            return dict(type=root + rel_to.path,
                display=unicode(value),
                data=root + rel_to.path + 'data/%s/' % value.pk)
    elif DATA_MAPPING.has_key(fieldmeta['type']):
        func = DATA_MAPPING[fieldmeta['type']]
        return func(model, instance, fieldmeta, value)
    else:
        if value is None:
            return None
        else:
            return unicode(value)


class _proxyEncoder(JSONEncoder):
    """If we don't know how to deal with the attribute type we'll just
    convert to a string and hope that's ok for now.
    """
    # An attribute inherited from JSONEncoder hide this method
    # pylint: disable=E0202
    def default(self, obj):
        return unicode(obj)


def as_json(_request, response, content_type):
    """Implement the default accept handling which will return JSON data.
    """
    response_root = getattr(response, 'root', None)
    if response_root:
        to_dump = response[response.root]
    else:
        to_dump = response
    if settings.DEBUG:
        dump_content = dumps(
            to_dump, indent=4,
            cls=_proxyEncoder)
    else:
        dump_content = dumps(
            to_dump, cls=_proxyEncoder)

    if content_type is not None and 'charset' not in content_type:
        content_type += '; charset=utf-8'

    return HttpResponse(
        dump_content, content_type or 'text/plain',
        status=response['_meta']['status'])



"""
    Implements the server side for the instance operators.
"""
from django.core.management.color import no_style
from django.db import connection

from slumber.operations import Operation
from slumber.server import get_slumber_root
from slumber.server.http import require_user, require_permission
from slumber.server.json import to_json_data


def _reset_sequence_number(model):
    """Private utility function to perform a SQL sequence number reset.
    """
    cursor = connection.cursor()
    reset_sequence_command_lines = connection.ops.sequence_reset_sql(
        no_style(), [model])
    if len(reset_sequence_command_lines) != 0:
        cursor.execute(';'.join(reset_sequence_command_lines))


def instance_data(into, model, instance):
    """Fill in the dict `into` with information about the instance of the
    specified model.
    """
    root = get_slumber_root()
    into['type'] = root + model.path
    into['identity'] = root + model.path + \
        '%s/%s/' % ('data', instance.pk)
    into['display'] = unicode(instance)
    into['operations'] = dict(
        [(op.name, op(instance))
            for op in model.operations.values() if not op.model_operation])
    into['fields'] = {}
    for field, meta in model.fields.items():
        into['fields'][field] = dict(
            data=to_json_data(model, instance, field, meta),
            kind=meta['kind'], type=meta['type'])
    into['data_arrays'] = {}
    for field in model.data_arrays:
        into['data_arrays'][field] = \
            into['identity'] + '%s/' % field


class InstanceData(Operation):
    """Return the instance data.
    """
    model_operation = False

    @require_user
    def get(self, request, response, pk, dataset=None):
        """Implement the fetching of attribute data for an instance.
        """
        instance = self.model.model.objects.get(pk=pk)
        if dataset:
            self._get_dataset(request, response, instance, dataset)
        else:
            self._get_instance_data(request, response, instance)

    def put(self, request, response, pk):
        """Allow a new version of the resource to be PUT here.
        """
        @require_permission('%s.add_%s' % (
                self.model.app, self.model.name.lower()))
        def do_put(_cls, request):
            """Apply the permission check to this inner function.
            """
            key_name = self.model.model._meta.pk.name
            pk_filter = {key_name: pk}
            created = (
                self.model.model.objects.filter(**pk_filter).count() == 0)

            instance = self.model.model(**dict(pk_filter.items() +
                [(k, v) for k, v in request.POST.items()]))
            instance.save()

            if created:
                response['_meta']['status'] = 201
                _reset_sequence_number(self.model.model)

            instance_data(response, self.model, instance)
            response['pk'] = to_json_data(self.model, instance, key_name,
                self.model.fields[key_name])

        return do_put(self, request)

    def delete(self, request, _response, pk):
        """Implement deletion of the instance.
        """
        @require_permission('%s.delete_%s' % (
                self.model.app, self.model.name.lower()))
        def do_delete(_cls, _request):
            """Apply the permission check to this inner function.
            """
            instance = self.model.model.objects.get(pk=pk)
            instance.delete()
        return do_delete(self, request)

    def _get_instance_data(self, _request, response, instance):
        """Return the base field data for the instance.
        """
        return instance_data(response, self.model, instance)

    def _get_dataset(self, request, response, instance, dataset):
        """Return one page of the array data.
        """
        root = get_slumber_root()
        response['instance'] = self(instance, dataset)

        try:
            query = getattr(instance, dataset + '_set')
        except AttributeError:
            query = getattr(instance, dataset)
        query = query.order_by('-pk')
        if request.GET.has_key('start_after'):
            query = query.filter(pk__lt=request.GET['start_after'])

        response['page'] = []
        for obj in query[:10]:
            model = type(obj).slumber_model
            response['page'].append(dict(
                    type=root + model.path,
                    pk=obj.pk, display=unicode(obj),
                    data=model.operations['data'](obj)))

        if query.count() > len(response['page']):
            response['next_page'] = self(instance, dataset,
                start_after=response['page'][-1]['pk'])




"""Implements the conversion of the response data to valid html data.
"""

import collections
from BeautifulSoup import BeautifulSoup
from django.conf import settings
from django.http import HttpResponse
from types import NoneType


# This module uses type and isinstance in an appropriate way
# pylint: disable=unidiomatic-typecheck


def build_html(_request, response, content_type):
    """Return http response object in text/html format.
    """
    html_template = '<!DOCTYPE HTML>\n<html><body>%s</body></html>'
    if hasattr(response, 'root'):
        dom = html_template % _convert(response[response.root])
    else:
        dom = html_template % _convert(response)
    if content_type is not None:
        if not 'charset' in content_type:
            content_type += '; charset=utf-8'
    if settings.DEBUG:
        dom = BeautifulSoup(dom).prettify()
    return HttpResponse(dom, content_type, status=response['_meta']['status'])


def _convert(obj):
    """This is the recursive converter.
    """
    if type(obj) in (int, float, str, unicode, bool, NoneType):
        return _convert_atom(obj)
    if isinstance(obj, dict):
        return _convert_dict(obj)
    if type(obj) in (list, set, tuple) or isinstance(obj, collections.Iterable):
        return _convert_list(obj)
    return _convert_atom(unicode(obj))


def _convert_atom(val):
    """Return xml element for atom which are 'int', 'float',
    'long', 'string', 'boolean' and 'null'.
    """
    if type(val) is int:
        val_type = "int"
    elif type(val) is float:
        val_type = "float"
    elif type(val) in (str, unicode):
        val_type = "string"
        if val.startswith('/') or val.startswith('http:'):
            val = '<a href="%s">%s</a>' % (val, val)
        else:
            val = "<br>".join(val.split("\n"))
    elif type(val) is bool:
        val_type = "boolean"
    elif type(val) is NoneType:
        val_type = "null"
        val = ""
    else:
        raise TypeError('Unsupported data type')
    return '<span class="%s">%s</span>' % (val_type, val)


def _convert_dict(obj):
    """Return xml element for dict.
    """
    output = "<dl>"
    for k, v in obj.items():
        output += '<dt>%s</dt>' % k
        output += '<dd>%s</dd>' % _convert(v)
    output += "</dl>"
    return output


def _convert_list(items):
    """Return xml element for list.
    """
    output = "<ol>"
    for item in items:
        output += '<li>%s</li>' % _convert(item)
    output += "</ol>"
    return output



#!/usr/bin/python
# -*- coding: utf-8 -*-
from BeautifulSoup import BeautifulSoup
from django.conf import settings
from django.test import TestCase
from mock import patch
import mock

from slumber.server.html import (build_html, _convert as convert,
    _convert_atom as convert_atom)
from slumber_examples.tests import ConfigureUser


class TestHTML(ConfigureUser, TestCase):
    def setUp(self):
        settings.DEBUG = True
        self.html_template = '<!DOCTYPE HTML>\n<html><body>%s</body></html>'
        super(TestHTML, self).setUp()

    @patch("slumber.server.html._convert")
    def test_as_HTML_should_call_convert (self, convert_mocked):
        # Arrange
        request = {}
        response = {'_meta': dict(status=200, message='OK')}
        content_type = None
        convert_mocked.return_value = 'text'
        expected_html_response = BeautifulSoup(self.html_template % convert_mocked.return_value).prettify()

        # Act
        response = build_html(request, response, content_type)

        # Assert
        self.assertTrue(convert_mocked.called)

        self.assertEqual(response.content, expected_html_response)

    @patch("slumber.server.html._convert")
    def test_as_HTML_should_append_charset_to_content_type(self, convert_mocked):
        # Arrange
        request = {}
        response = {'_meta': dict(status=200, message='OK')}
        content_type = 'text/html'
        convert_mocked.return_value = 'text'

        # Act
        response = build_html(request, response, content_type)

        # Assert
        self.assertEqual(response['Content-Type'], 'text/html; charset=utf-8')

    @patch("slumber.server.html._convert")
    def test_as_HTML_should_not_append_charset_to_content_type_if_there_is_one(self, convert_mocked):
        # Arrange
        request = {}
        response = {'_meta': dict(status=200, message='OK')}
        content_type = 'text/html; charset=utf-8'
        convert_mocked.return_value = 'text'

        # Act
        response = build_html(request, response, content_type)

        # Assert
        self.assertEqual(response['Content-Type'], 'text/html; charset=utf-8')

    def test_convert_with_int_type(self):
        # Arrange
        test_dict = {'a':5}
        expect_result = '<dl><dt>a</dt><dd><span class="int">5</span></dd></dl>'

        # Act
        result = convert(test_dict)

        # Assert
        self.assertEqual(result, expect_result)

    def test_convert_with_float_type(self):
        # Arrange
        test_dict = {'a':5.55}
        expect_result = '<dl><dt>a</dt><dd><span class="float">5.55</span></dd></dl>'

        # Act
        result = convert(test_dict)

        # Assert
        self.assertEqual(result, expect_result)

    def test_convert_with_string_type__convert_backslash_into_br_tag(self):
        # Arrange
        test_dict = {'a':'Test_text\n'}
        expect_result = '<dl><dt>a</dt><dd><span class="string">Test_text<br></span></dd></dl>'
        # Act
        result = convert(test_dict)
        # Assert
        self.assertEqual(result, expect_result)

    def test_convert_with_array_type(self):
        # Arrange
        test_dict = {'a':[1, 2, 3]}
        expect_result = '<dl><dt>a</dt><dd><ol><li><span class="int">1</span></li><li><span class="int">2</span></li>' \
                        '<li><span class="int">3</span></li></ol></dd></dl>'

        # Act
        result = convert(test_dict)

        # Assert
        self.assertEqual(result, expect_result)

    def test_convert_with_none_type(self):
        # Arrange
        test_dict = {'a':None}
        expect_result = '<dl><dt>a</dt><dd><span class="null"></span></dd></dl>'

        # Act
        result = convert(test_dict)

        # Assert
        self.assertEqual(result, expect_result)

    def test_convert_with_boolean_type(self):
        # Arrange
        test_dict = {'a':True}
        expect_result = '<dl><dt>a</dt><dd><span class="boolean">True</span></dd></dl>'

        # Act
        result = convert(test_dict)

        # Assert
        self.assertEqual(result, expect_result)

    def test_convert_atom_with_Mock_type(self):
        # Arrange
        test_dict = mock

        # Assert
        with self.assertRaises(TypeError) as e:
            convert_atom(test_dict)

        self.assertEqual(e.exception.message, 'Unsupported data type')

    def test_convert_atom_with_unicode_input(self):
        # Arrange
        test_dict = {'a': u'\u2014'}
        expect_result = u'<dl><dt>a</dt><dd><span class="string">—</span></dd></dl>'

        # Act
        result = convert(test_dict)

        # Assert
        self.assertEqual(result, expect_result)




""" Dispatch type of http accept header handler
"""
from slumber.server.html import build_html as as_html
from slumber.server.json import as_json
from slumber.server.xml import as_xml


def get_handlers_list():
    """The default set of content encoders.
    """
    return [
        ('application/json', as_json),
        ('text/html', as_html),
        ('text/xml', as_xml),
        ('application/xml', as_xml),
    ]


def accept(accept_header, accept_handlers_list=None):
    """Perform the content type negotiation.
    """
    if not accept_handlers_list:
        accept_handlers_list = get_handlers_list()
    for accept_str, fn_handler in accept_handlers_list:
        if accept_str in accept_header:
            return accept_str, fn_handler

    return None, as_json



from slumber import configure
from slumber.operations.instancedata import InstanceData
from slumber.operations.instancelist import InstanceListHal

from models import Pizza, Shop
from operations import OrderPizza, ShopList


configure(
    {'test': True})

configure(Pizza,
    operations_extra = [(OrderPizza, 'order')])

configure(Shop,
    operations_extra = [
        (None, 'delete'),
        (ShopList, 'shops1', 'shops/mount1'),
        (InstanceListHal, 'shops-hal', 'shops/mount2'),
        (InstanceData, 'data', 'pizzas/shop'),
    ],
    properties_ro = ['web_address'])



from django.test import TestCase

from slumber.connector.ua import get

from slumber_examples.models import Shop
from slumber_examples.tests.configurations import ConfigureUser

class TestJSON(ConfigureUser, TestCase):

    def setUp(self):
        super(TestJSON, self).setUp()
        self.old_value = settings.DEBUG
        settings.DEBUG = True

    def tearDown(self):
        settings.DEBUG = self.old_value
        super(TestJSON, self).tearDown()

    def test_as_json(self):
        # Arrange
        request = {}
        response = {'_meta': dict(status=200, message='OK'),
                    'fake_content': 'sputnik'}
        content_type = 'application/json'

        expected_content_type = "%s; charset=utf-8" % content_type
        expected_response = HttpResponse(request, expected_content_type, 200)

        # Act
        http_response = as_json(request, response, content_type)

        # Assert
        self.assertEqual(str(http_response), str(expected_response))


    def test_as_json_should_not_append_charset_if_its_provided(self):
        # Arrange
        request = {}
        response = {'_meta': dict(status=200, message='OK'),
                    'fake_content': 'sputnik'}
        content_type = 'application/json; charset=utf-16'

        # Act
        http_response = as_json(request, response, content_type)

        # Assert
        self.assertEqual(http_response['Content-Type'], 'application/json; charset=utf-16')



"""
    A test version of the client.
"""
import mock

from slumber.connector.api import get_instance, get_model_type
from slumber.connector.dictobject import DictObject


class _Operations(object):
    """Allows us to fetch URLs for operations in mocks reliably.
    """
    def __init__(self, url):
        self._url = 'http://' + url
        self._suffix = '/'

    def __getitem__(self, name):
        return self._url + name + self._suffix


class _MockModel(object):
    """A mock model object type so we can attach things more sanely for
    mocking purposes.
    """
    def __init__(self, client, url, instance_type):
        self.client = client
        self._url = url
        self._operations = _Operations(url)
        self.instance_type = instance_type
        self.instances = []

    def __call__(self, url, display_name):
        return get_instance('slumber://' + self._url, url, display_name)

    def get(self, **query):
        """Implements a mocked version of the get operator.
        """
        for i in self.instances:
            found = True
            for k, v in query.items():
                found = found and hasattr(i, k) and (
                    getattr(i, k) == v or unicode(getattr(i, k)) == unicode(v))
            if found:
                if hasattr(i, '_url'):
                    return get_instance(
                        'slumber://' + self._url, i._url, None)
                else:
                    return i
        assert False, "The instance was not found\n%s" % query

    def create(self, **items):
        """Implements a mocked version of the create operator.
        """
        instance = self.instance_type(self._url, **items)
        self.instances.append(instance)
        self.client._instances.append(instance)
        return instance

    def update(self, instance_connector, **updating_data):
        """ Implement a mocked version of update operator """
        updating_id = instance_connector.pk

        for instance in self.instances:
            if instance.pk == updating_id:
                for key, value in updating_data.items():
                    setattr(instance, key, value)


class _MockInstance(DictObject):
    """A mock instance that will add in a _url parameter.
    """
    def __init__(self, model_url, **kwargs):
        super(_MockInstance, self).__init__(**kwargs)
        self._operations = _Operations(model_url)
        if hasattr(self, 'pk') or hasattr(self, 'id'):
            pk = getattr(self, 'pk', getattr(self, 'id', None))
            self._operations._suffix = '/%s/' % pk
            self._url = ('slumber://' + model_url +
                'data%s' % self._operations._suffix)

    def __repr__(self):
        return getattr(self, '_url', 'Unknown mock instance')


class _MockClient(DictObject):
    """Mock slumber client class.
    """
    def __init__(self, **instances):
        super(_MockClient, self).__init__()
        self._instances = []
        for model, instances in instances.items():
            root = self
            for k in model.split('__')[:-1]:
                if not hasattr(root, k):
                    setattr(root, k, DictObject())
                root = getattr(root, k)
            model_name = model.split('__')[-1]
            model_url = model.replace('__', '/') + '/'
            model_type = get_model_type(model_url, [_MockModel])
            instance_type = type(model_url, (_MockInstance,), {})
            mock_model = model_type(self, model_url, instance_type)
            setattr(root, model_name, mock_model)
            for instance in instances:
                mock_model.create(**instance)

    def _flush_client_instance_cache(self):
        """Empty stub so that the middleware works in tests.
        """


def mock_client(**instances):
    """Replaces the client with a mocked client that provides access to the
    provided applications, models and instances.
    """
    models = _MockClient(**instances)
    def decorator(test_method):
        """The actual decorator that is going to be used on the test method.
        """
        @mock.patch('slumber._client', models)
        def test_wrapped(test, *a, **kw):
            """The wrapper for the test method.
            """
            test_method(test, *a, **kw)
        test_wrapped.__doc__ = test_method.__doc__
        return test_wrapped
    return decorator


def mock_ua(test_method):
    """Allow the user agent to set up expectations.
    """
    class MockUA(object):
        """Records the expectations and check that they are met properly.
        """
        def __init__(self, test):
            self.expectations = []
            self.test = test

        def get(self, url, data):
            """Record a get expectation.
            """
            self.expectations.append(('get', url, None, data))

        def post(self, url, edata, rdata):
            """Record a post expectation.
            """
            self.expectations.append(('post', url, edata, rdata))

        def do_get(self, url, _ttl=0, _codes=None, _headers=None):
            """The patch for the user agent get.
            """
            self.test.assertTrue(self.expectations,
                "No expectation for GET %s" % url)
            emethod, eurl, edata, rdata = self.expectations.pop(0)
            self.test.assertEqual(emethod, 'get')
            self.test.assertEqual(url, eurl)
            self.test.assertIsNone(edata)
            return None, rdata

        def do_post(self, url, data, _codes=None):
            """The patch for the user agent post.
            """
            self.test.assertTrue(self.expectations,
                "No expectation for POST %s\n%s" % (url, data))
            emethod, eurl, edata, rdata = self.expectations.pop(0)
            self.test.assertEqual(emethod, 'post')
            self.test.assertEqual(url, eurl)
            if not edata is None:
                self.test.assertEqual(data, edata)
            return None, rdata

    def test_wrapped(test, *a, **kw):
        """Test decorator that patches the user agent and ensures all
        expectations are met.
        """
        user_agent = MockUA(test)
        with mock.patch('slumber.connector.ua._get', user_agent.do_get):
            with mock.patch('slumber.connector.ua._post', user_agent.do_post):
                test_method(test, user_agent, *a, **kw)
                test.assertEqual(user_agent.expectations, [],
                    "There are un-used expectations")
    test_wrapped.__doc__ = test_method.__doc__
    return test_wrapped




from mock import patch
from simplejson import loads

from django.test import TestCase

from slumber import Client
from slumber.connector.ua import post, get, put

from slumber_examples.models import Pizza, PizzaCrust, Shop
from slumber_examples.tests.configurations import ConfigureUser


class CreateTests(ConfigureUser, TestCase):
    def test_create_pizza(self):
        response, json = post('/slumber/slumber_examples/Pizza/create/', {
            'id': 1, 'name': 'Test P', 'for_sale': True})
        self.assertEqual(response.status_code, 200)
        self.assertTrue(json.has_key('pk'), json)
        self.assertEqual(json['pk'], 1, json)
        self.assertTrue(json.has_key('created'), json)
        self.assertTrue(json['created'], json)
        self.assertEqual(Pizza.objects.all().count(), 1)

    def test_put_pizza_infer_id(self):
        response, json = put('/slumber/slumber_examples/Pizza/data/123/',
            {'name': 'Test P', 'for_sale': True})
        self.assertEqual(response.status_code, 201)
        self.assertTrue(json['fields'].has_key('id'), json)
        self.assertEqual(json['fields']['id']['data'], '123', json)
        self.assertEqual(Pizza.objects.all().count(), 1)

    def test_create_pizza_twice(self):
        response1, json1 = post('/slumber/slumber_examples/Pizza/create/', {
                'id': 1, 'name': 'Test P', 'for_sale': True})
        self.assertEqual(response1.status_code, 200)
        self.assertTrue(json1['created'], json1)

        response2, json2 = post('/slumber/slumber_examples/Pizza/create/', {
                'id': 1, 'name': 'Test P', 'for_sale': True})
        self.assertEqual(response2.status_code, 200)
        self.assertFalse(json2['created'], json2)
        self.assertEqual(Pizza.objects.all().count(), 1)

    def test_create_pizza_crust(self):
        response1, json1 = post('/slumber/slumber_examples/PizzaCrust/create/', {
                'code': 'org', 'full_name': 'original crust'})
        self.assertEqual(response1.status_code, 200)
        self.assertTrue(json1['created'], json1)
        self.assertEqual(PizzaCrust.objects.all().count(), 1)

    def test_update_pizza(self):
        self.cnx = Client()
        p_id, p_sale = 1 , False
        response1, json1 = post('/slumber/slumber_examples/Pizza/create/',{
                'id':p_id, 'name':'Test P', 'for_sale': True})
        self.assertEqual(response1.status_code, 200)
        pizza = self.cnx.slumber_examples.Pizza.get(pk = p_id)
        self.cnx.slumber_examples.Pizza.update(pizza, for_sale = p_sale)
        pizza = self.cnx.slumber_examples.Pizza.get(pk = p_id)
        self.assertEqual(pizza.for_sale, p_sale)


class OrderTests(ConfigureUser, TestCase):
    def setUp(self):
        super(OrderTests, self).setUp()
        self.pizza = Pizza(name='S1', for_sale=True)
        self.pizza.save()
        self.cnx = Client()

    def test_order_get(self):
        with patch('slumber.connector._get_slumber_authn_name', lambda: 'service'):
            pizza = self.cnx.slumber_examples.Pizza.get(id=self.pizza.id)
            self.assertEquals(pizza._operations['order'],
                'http://localhost:8000/slumber/slumber_examples/Pizza/order/1/')
            # Do an unsigned GET so there will be no user logged in
            response = self.client.get('/slumber/slumber_examples/Pizza/order/1/')
            self.assertEquals(response.status_code, 200, response.content)
            json = loads(response.content)
            self.assertEquals(json, dict(
                    _meta = dict(status=200, message="OK"),
                    form = dict(quantity='integer'),
                ))

    def test_order_post(self):
        pizza = self.cnx.slumber_examples.Pizza.get(id=self.pizza.id)
        response = self.client.post('/slumber/slumber_examples/Pizza/order/1/',
            dict(quantity=3))
        self.assertEquals(response.status_code, 501, response.content)

    def test_order_create_with_pk(self):
        pizza2 = self.cnx.slumber_examples.Pizza.create(id=2, name='S2', for_sale=True)
        pizza3 = Pizza.objects.create(name='S3', for_sale=True)
        self.assertEqual(Pizza.objects.count(), 3)


class ShopListTests(TestCase):
    def setUp(self):
        super(ShopListTests, self).setUp()
        self.cnx = Client()

    def test_mount_point(self):
        response, json = get('/slumber/shops/mount1/')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json, dict(_meta={'message': 'OK', 'status': 200},
            shops=[{
                'name': 'Hard Coded Pizza Parlour'
            }]))

    def test_operation_model_uri(self):
        self.assertTrue(
            self.cnx.slumber_examples.Shop._operations.has_key('shops1'),
            self.cnx.slumber_examples.Shop._operations.keys())
        self.assertEqual(
            self.cnx.slumber_examples.Shop._operations["shops1"],
            'http://localhost:8000/slumber/shops/mount1/')


class ShopInstanceTests(ConfigureUser, TestCase):
    def test_mount_point(self):
        shop = Shop.objects.create(name='Test One')
        with patch('slumber.connector._get_slumber_authn_name', lambda: 'service'):
            response, json = get('/slumber/pizzas/shop/%s/' % shop.pk)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(json['operations'].has_key('data'), json)
        self.assertEqual(json['operations']['data'],
            '/slumber/pizzas/shop/%s/' % shop.pk)





from __future__ import absolute_import

from django.test import TestCase
from xml.dom.minidom import parseString

from slumber.connector.ua import get

from slumber_examples.models import Shop
from slumber_examples.tests.configurations import ConfigureUser


class TestInstanceList(ConfigureUser, TestCase):
    def test_empty_hal_version(self):
        response, json = get('/slumber/shops/mount2/')

        self.assertEqual(response.status_code, 200)
        self.assertFalse(json.has_key('instances'))

        self.assertTrue(json.has_key('_links'))
        links = json['_links']
        self.assertEqual(links['self']['href'], '/slumber/shops/mount2/')
        self.assertEqual(links['model']['href'], '/slumber/slumber_examples/Shop/')

    def test_five_shops(self):
        for s in xrange(1, 6):
            Shop.objects.create(name="Shop %d" % s, slug="shop%d" % s)
        response, json = get('/slumber/shops/mount2/')

        self.assertEqual(response.status_code, 200)
        self.assertFalse(json.has_key('instances'))

        self.assertTrue(json.has_key('_links'))
        links = json['_links']
        self.assertEqual(links['self']['href'], '/slumber/shops/mount2/')
        self.assertEqual(links['model']['href'], '/slumber/slumber_examples/Shop/')
        self.assertFalse(links.has_key('next'), links)

        self.assertTrue(json.has_key('_embedded'))
        self.assertTrue(json['_embedded'].has_key('page'))
        page = json['_embedded']['page']

        self.assertEqual(len(page), 5)
        self.assertEqual(page[0], dict(
            _links={'self': dict(href='/slumber/pizzas/shop/5/')},
            display="Shop 5"))

    def test_ten_shops(self):
        for s in xrange(1, 11):
            Shop.objects.create(name="Shop %d" % s, slug="shop%2d" % s)
        response, json = get('/slumber/shops/mount2/')

        self.assertEqual(response.status_code, 200)
        self.assertFalse(json.has_key('instances'))

        self.assertTrue(json.has_key('_links'))
        links = json['_links']
        self.assertEqual(links['self']['href'], '/slumber/shops/mount2/')
        self.assertEqual(links['model']['href'], '/slumber/slumber_examples/Shop/')
        self.assertFalse(links.has_key('next'), links)

        self.assertTrue(json.has_key('_embedded'))
        self.assertTrue(json['_embedded'].has_key('page'))
        page = json['_embedded']['page']

        self.assertEqual(len(page), 10)
        self.assertEqual(page[0], dict(
            _links={'self': dict(href='/slumber/pizzas/shop/10/')},
            display="Shop 10"))

    def test_fifteen_shops(self):
        for s in xrange(1, 16):
            Shop.objects.create(name="Shop %d" % s, slug="shop%2d" % s)
        response, json = get('/slumber/shops/mount2/')

        self.assertEqual(response.status_code, 200)
        self.assertFalse(json.has_key('instances'))

        self.assertTrue(json.has_key('_links'))
        links = json['_links']
        self.assertEqual(links['self']['href'], '/slumber/shops/mount2/')
        self.assertEqual(links['model']['href'], '/slumber/slumber_examples/Shop/')
        self.assertTrue(links.has_key('next'), links)

        self.assertTrue(json.has_key('_embedded'))
        self.assertTrue(json['_embedded'].has_key('page'))
        page = json['_embedded']['page']

        self.assertEqual(len(page), 10)
        self.assertEqual(page[0], dict(
            _links={'self': dict(href='/slumber/pizzas/shop/15/')},
            display="Shop 15"))

        self.assertEqual(links['next']['href'],
            '/slumber/shops/mount2/?lpk=6')

    def test_fifteen_shops_page_2(self):
        for s in xrange(1, 16):
            Shop.objects.create(name="Shop %d" % s, slug="shop%2d" % s)
        response, json = get('/slumber/shops/mount2/?lpk=6')
        self.assertEqual(response.status_code, 200)
        self.assertFalse(json.has_key('instances'))

        self.assertTrue(json.has_key('_links'))
        links = json['_links']
        self.assertFalse(links.has_key('next'), links)

        self.assertTrue(json.has_key('_embedded'))
        self.assertTrue(json['_embedded'].has_key('page'))
        page = json['_embedded']['page']

        self.assertEqual(len(page), 5)
        self.assertEqual(page[0], dict(
            _links={'self': dict(href='/slumber/pizzas/shop/5/')},
            display="Shop 5"))

    def test_xml(self):
        response, _ = get('/slumber/shops/mount2/?lpk=6',
            headers=dict(Accept='application/xml'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/xml; charset=utf-8')

        xml = parseString(response.content)
        self.assertEqual(xml.documentElement.tagName, 'instances')



"""
    Implements configuration of the Slumber models available on the server.
"""
from django.core.urlresolvers import reverse

from slumber._caches import DJANGO_MODEL_TO_SLUMBER_MODEL, \
    OPERATION_URIS
from slumber.connector.configuration import INSTANCE_PROXIES, MODEL_PROXIES
from slumber.server.json import DATA_MAPPING
from slumber.server.meta import get_application


def configure(arg,
        properties_ro=None,
        to_json=None,
        operations_extra=None,
        instance_proxy=None,
        model_proxy=None):
    """Configure Slumber for the provided model.

    When configuring the server side the model is a model instance. When
    configuring the client the model needs to be the tail of the model path.

    Server configuration:

    * properties_ro : A list of properties that may be read from the client,
        but not set
    * to_json : A dict giving functions by type name which transform the
        property value to a JSON compatible Python object.
        See slumber/server/json.py for the function signatures and some
        examples
    * operations_extra: A list of operations that are to be added to the
        model.

    Client configuration:

    * instance_proxy: The proxy to be used on the client side when instances
    of this model are created.
    * model_proxy: The proxy to be used on the client side when this model is
        encountered.
    """
    # We need all of these arguments as they are all used
    # pylint: disable=R0913
    if isinstance(arg, basestring):
        _model_name(arg, instance_proxy, model_proxy)
    elif isinstance(arg, dict):
        _configuration(arg)
    else:
        _model(arg, to_json, properties_ro, operations_extra)


def _model_name(model_name, instance_proxy, model_proxy):
    """Process configuration given by a Django model name.
    """
    if instance_proxy:
        INSTANCE_PROXIES[model_name] = instance_proxy
    if model_proxy:
        MODEL_PROXIES[model_name] = model_proxy


def _configuration(config):
    """Process a configuratoin that is to be added to the service meta-data.
    """
    from slumber.server.meta import IMPORTING
    assert IMPORTING, "Slumber itself must call configure"
    app = get_application(IMPORTING)
    app.configuration = config


def _model(django_model, to_json, properties_ro, operations_extra):
    """Process configuration for a Django model
    """
    model = DJANGO_MODEL_TO_SLUMBER_MODEL[django_model]

    model.properties['r'] += properties_ro or []
    for type_name, function in (to_json or {}).items():
        DATA_MAPPING[type_name] = function

    for conf in operations_extra or []:
        if len(conf) == 2:
            operation, name = conf
            if operation is None:
                del model.operations[name]
            else:
                model.operations[name] = operation(model, name)
        elif len(conf) == 3:
            operation, name, uri = conf
            slumber_op = operation(model, name,
                reverse('slumber.server.views.service_root') + uri + '/')
            model.operations[name] = slumber_op
            OPERATION_URIS[uri] = slumber_op
